/* Copyright © 2010 Brandon L Black <blblack@gmail.com>
 *
 * This file is part of gdnsd.
 *
 * gdnsd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * gdnsd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with gdnsd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "conf.h"
#include "dlsym_fptr.h"
#include "monio.h"
#include "dnsio_udp.h"
#include "dnsio_tcp.h"

#include <unistd.h>
#include <string.h>
#include <libgen.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <unistd.h>
#include <dlfcn.h>
#include <limits.h>

#ifndef INET6_ADDRSTRLEN
#define INET6_ADDRSTRLEN 46
#endif

static const char def_pidfile[] = VARDIR "/run/" PACKAGE_NAME ".pid";
static const char def_username[] = PACKAGE_NAME;
static const char def_chroot_path[] = VARDIR "/" PACKAGE_NAME;

// Global config, readonly after loaded from conf file
global_config_t gconfig = {
    .zones = NULL,
    .dns_addrs = NULL,
    .http_addrs = NULL,
    .pidfile = def_pidfile,
    .username = def_username,
    .chroot_path = def_chroot_path,
    .curve_keys = NULL,
    .num_curve_keys = 0,
    .include_optional_ns = false,
    .realtime_stats = false,
    .lock_mem = false,
    .disable_text_autosplit = false,
    .strict_data = true,
     // legal values are -20 to 20, so -21
     //  is really just an indicator that the user
     //  didn't explicitly set it.  The default
     //  behavior is dynamic...
    .priority = -21,
    .zones_default_ttl = 86400U,
    .log_stats = 3600U,
    .max_http_clients = 128U,
    .http_timeout = 5U,
    .num_zones = 0U,
    .num_dns_addrs = 0U,
    .num_http_addrs = 0U,
    .max_response = 16384U,
    .max_cname_depth = 16,
};

// Generic iterator for catching bad config hash keys in various places below
F_NONNULL
static bool bad_key(const char* key, unsigned klen V_UNUSED, const vscf_data_t* d V_UNUSED, void* data) {
    assert(data); assert(key);
    log_fatal("Invalid %s key '%s'", (const char*)data, key);
}

F_MALLOC F_WUNUSED F_NONNULL
static char* absify(const char* inpath) {
    assert(inpath);
    if(*inpath == '/') return strdup(inpath);

    char* out = malloc(PATH_MAX);
    if(!getcwd(out, PATH_MAX))
        log_fatal("getcwd() failed: %s", logf_errno());

    size_t cwdlen = strlen(out);
    size_t inlen = strlen(inpath);
    size_t final = cwdlen + inlen + 2;

    if(final >= PATH_MAX)
        log_fatal("Fully-qualified config pathname exceeds PATH_MAX");

    out = realloc(out, final);
    out[cwdlen] = '/';
    memcpy(out + cwdlen + 1, inpath, inlen + 1);

    return out;
}

// Translate listen-address specification to an anysin_t stored at result.
//  result is already allocated and zerod.  listen-address specs are ASCII
//  strings of the form IP@Port, where the IP part is required and can be
//  a v4 or v6 address in standard form, and the @Port part is optional.
F_NONNULL
static void make_addr(const char* txt, anysin_t* result, unsigned def_port) {
    assert(txt); assert(result); assert(def_port);

    char addrcopy[INET6_ADDRSTRLEN + 1];
    const char* addr_txt;
    unsigned port = def_port;

    // IP@port
    char* at_ptr;
    if((at_ptr = strchr(txt, '@'))) {
        // Copy out the address part
        uintptr_t at_offs = at_ptr - txt;
        if(at_offs < 2 || at_offs > INET6_ADDRSTRLEN)
            log_fatal("Failed to parse IP[@Port] spec '%s'", txt);
        memcpy(addrcopy, txt, at_offs);
        addrcopy[at_offs] = '\0';
        addr_txt = addrcopy;

        // Parse the port part
        at_ptr++;
        errno = 0;
        long pnum = strtol(at_ptr, NULL, 10);
        if(errno)
            log_fatal("Failed to parse port number '%s' in listen spec '%s': %s", at_ptr, txt, logf_errno());
        if(pnum < 1 || pnum > 65535)
            log_fatal("Port number '%s' out of range in listen spec '%s'", at_ptr, txt);
        port = pnum;
    }
    // Just an IP, use def_port
    else {
        addr_txt = txt;
    }

    int status;

    if(!strchr(addr_txt, ':')) { // No colons == IPv4
        result->sa.sa_family = AF_INET;
        status = inet_pton(AF_INET, addr_txt, &result->sin.sin_addr);
        result->sin.sin_port = htons(port);
    }
    else { // Colons == assumed IPv6
#if USE_V6SOCK
        result->sa.sa_family = AF_INET6;
        status = inet_pton(AF_INET6, addr_txt, &result->sin6.sin6_addr);
        result->sin6.sin6_port = htons(port);
#else
        log_fatal("This build does not support IPv6 for listen spec '%s'", txt);
#endif
    }

    if(status <= 0)
        log_fatal("IP@Port address specification '%s' invalid", txt);
}

#if DNSCURVE

F_NONNULLX(1)
static void make_key_data(const char* keys_dir, const vscf_data_t* ck_cfg) {
    assert(keys_dir);

    if(!ck_cfg)
        return;

    if(!vscf_is_hash(ck_cfg))
        log_fatal("curve_keys must be a hash");

    unsigned count;
    if(!(count = vscf_hash_get_len(ck_cfg)))
        return;

    gconfig.num_curve_keys = count;
    curvekey_t* keys = gconfig.curve_keys = calloc(count, sizeof(curvekey_t));
    for(unsigned i = 0; i < count; i++) {
        curvekey_t* this_key = &keys[i];
        const char* key_name = vscf_hash_get_key_byindex(ck_cfg, i, NULL);
        this_key->name = strdup(key_name);
        const vscf_data_t* key_cfg_data = vscf_hash_get_data_byindex(ck_cfg, i);
        if(!vscf_is_hash(key_cfg_data))
            log_fatal("Value for curve_key '%s' must be a hash", key_name);

        const vscf_data_t* key_esep_bits;
        if((key_esep_bits = vscf_hash_get_data_byconstkey(key_cfg_data, "nonce_sep_bits", true))) {
            unsigned long raw_esep_bits;
            if(!vscf_is_simple(key_esep_bits) || !vscf_simple_get_as_ulong(key_esep_bits, &raw_esep_bits))
                log_fatal("Attribute 'nonce_sep_bits' of curve_key '%s' must be an unsigned integer <= 32", key_name);
            if(raw_esep_bits > 32U)
                log_fatal("Attribute 'nonce_sep_bits' of curve_key '%s' must be an unsigned integer <= 32", key_name);
            this_key->esep_bits = raw_esep_bits;
        }

        const vscf_data_t* key_esep_val;
        if((key_esep_val = vscf_hash_get_data_byconstkey(key_cfg_data, "nonce_sep_val", true))) {
            unsigned long raw_esep_val;
            if(!vscf_is_simple(key_esep_val) || !vscf_simple_get_as_ulong(key_esep_val, &raw_esep_val))
                log_fatal("Attribute 'nonce_sep_val' of curve_key '%s' must be an unsigned integer", key_name);
            if(raw_esep_val > ((1U << this_key->esep_bits) - 1U))
                log_fatal("curve_key '%s': nonce_sep_val '%lu' does not fit within nonce_sep_bits '%u'", key_name, raw_esep_val, this_key->esep_bits);
            this_key->esep_val = raw_esep_val;
        }

        const vscf_data_t* key_cfg_file;
        const char* fn;
        if((key_cfg_file = vscf_hash_get_data_byconstkey(key_cfg_data, "file", true))) {
            if(!vscf_is_simple(key_cfg_file))
                log_fatal("Attribute 'file' of curve_key '%s' must be a simple string", key_name);
            fn = make_cf_fn(keys_dir, vscf_simple_get_data(key_cfg_file));
        }
        else {
            fn = make_cf_fn(keys_dir, key_name);
        }

        vscf_hash_iterate(key_cfg_data, true, bad_key, "per-key curve_keys attribute");

        this_key->file = fn;

        // We delay reading the key files until the time
        //  window between daemonization and chroot(), which
        //  is the only time we can make mlock() persist while
        //  still being able to read filesystem files.
        // However, we check w/ lstat() and open() here just
        //  to provide an earlier failure for the common problem
        //  cases while stderr is still open.
        struct stat fnstat;
        if(lstat(fn, &fnstat))
            log_fatal("lstat() of file '%s' for curve key '%s' failed: %s", fn, key_name, logf_errno());
        if((fnstat.st_mode & S_IFMT) != S_IFREG)
            log_fatal("file '%s' for curve key '%s' is not a regular file", fn, key_name);
        if(fnstat.st_size != 32)
            log_fatal("file '%s' for curve key '%s' is not exactly 32 bytes long", fn, key_name);

        // While we're at it, warn about permissive perms
        if((fnstat.st_mode & 0777) != 0400)
            log_warn("file '%s' for curve key '%s' has insecure access permissions.  You should correct this with 'chmod 400 %s'", fn, key_name, fn);

        int fd = open(fn, O_RDONLY|O_NOFOLLOW);
        if(fd < 0)
            log_fatal("Cannot open() file '%s' for curve key '%s': %s", fn, key_name, logf_errno());
        close(fd);
    }
}

// count2bits turns an unsigned count of items into
//  the number of bits it takes to assign each
//  item a unique number.  i.e.
//  0 -> 0, 1 -> 0, 2 -> 1, 3 -> 2, 4 -> 2, 5 -> 3, etc...

#ifndef HAVE_BUILTIN_CLZ

F_CONST
static inline unsigned count2bits(unsigned x) {
    if(x < 2)
        return 0;

    x--;
    unsigned nbits = 0;
    do {
        nbits++;
        x >>= 1;
    } while(x);

    return nbits;
}

#else

F_CONST
static inline unsigned count2bits(unsigned x) {
    if(x < 2)
        return 0;
    return 32U - __builtin_clz(x - 1);
}

#endif

// Calculate the per-key sep_mask (iterates gconfig.curve_keys, uses the initial
//  config values from make_key_data() combined with the num_socks values set
//  above in process_listen()).
static void calc_sep_masks(void) {
    unsigned n_keys = gconfig.num_curve_keys;
    for(unsigned i = 0; i < n_keys; i++) {
        curvekey_t* this_key = &gconfig.curve_keys[i];
        unsigned bits = this_key->esep_bits + count2bits(this_key->num_socks);
        // should never happen in any sane real-world configuration...
        if(bits > 32U)
            log_fatal("Total nonce sep bits (configured + internal per-socket) for curve_key '%s' exceeds the limit of 32", this_key->name);
        uint32_t ones_mask = 0;
        while(bits--) {
            ones_mask >>= 1;
            ones_mask |= 0x80000000;
        }
        // invert the mask above, i.e. if bits is 3, the final
        //  mask should have the top 3 bits clear and the rest 1's.
        //  (the mask is anded with the original top 32 bits of the nonce
        //   before the sep_val is or'd into place)
        this_key->sep_mask = ~ones_mask;
        // log_debug("Key #%u - name '%s' - sep_mask %x", i, this_key->name, this_key->sep_mask);
    }
}

// Calculate the per-socket sep_val (iterates the dns_addrs, follows references
//   into gconfig.curve_keys, uses the previously calculated num_socks and sep_mask
//   values, also maintains and uses cur_sock to know the final local value).
static void calc_sep_vals(void) {
    unsigned n_addrs = gconfig.num_dns_addrs;
    for(unsigned i = 0; i < n_addrs; i++) {
        dns_addr_t* this_addr = &gconfig.dns_addrs[i];
        unsigned n_keys = this_addr->num_curve_keys;
        for(unsigned j = 0; j < n_keys; j++) {
            dnsaddr_key_t* this_dkey = &this_addr->curve_keys[j];
            curvekey_t* this_key = this_dkey->key;

            // Get the shift count for the esep value
            unsigned esep_bits = this_key->esep_bits;
            unsigned esep_shift = esep_bits ? (32U - esep_bits) : 0;

            // Get the shift count for the isep value
            unsigned isep_bits = count2bits(this_key->num_socks);
            unsigned isep_allbits = isep_bits + esep_bits;
            unsigned isep_shift = isep_allbits ? (32U - isep_allbits) : 0;

            // set local esep_val to the shifted esep value
            uint32_t esep_val = this_key->esep_val;
            esep_val <<= esep_shift;

            // UDP: shift the isep val (cur_sock-based),
            //  combine with esep_val for final udp_sep_val
            uint32_t udp_isep_val = this_key->cur_sock++;
            assert(udp_isep_val < (1U << isep_bits));
            udp_isep_val <<= isep_shift;
            this_dkey->udp_sep_val = esep_val | udp_isep_val;

            // log_debug("addr %u key %u udp_sep_val %x", i, j, this_dkey->udp_sep_val);

            // or and xor should produce identical results since the values
            //  should not have any overlapping 1's:
            assert(this_dkey->udp_sep_val == (esep_val ^ udp_isep_val));
            // and the final value should fit within the mask
            assert((this_dkey->udp_sep_val & this_key->sep_mask) == 0);

            // Repeat for TCP is applicable
            if(!this_addr->tcp_disabled) {
                uint32_t tcp_isep_val = this_key->cur_sock++;
                assert(tcp_isep_val < (1U << isep_bits));
                tcp_isep_val <<= isep_shift;
                this_dkey->tcp_sep_val = esep_val | tcp_isep_val;
                // log_debug("addr %u key %u tcp_sep_val %x", i, j, this_dkey->tcp_sep_val);
                assert(this_dkey->tcp_sep_val == (esep_val ^ tcp_isep_val));
                assert((this_dkey->tcp_sep_val & this_key->sep_mask) == 0);
            }
        }
    }
}

#endif // DNSCURVE

// for zones_dir, curve_keys_dir, etc
F_NONNULLX(1)
static char* configure_cfdir(const char* cfg_dir, const char* opt) {
    assert(cfg_dir);

    char* retval;

    if(opt && *opt == '/') { // absolute path provided
        size_t opt_len = strlen(opt);
        if(opt[opt_len - 1] == '/') {
            retval = strdup(opt);
        }
        else {
            char* zdir = malloc(opt_len + 2);
            memcpy(zdir, opt, opt_len);
            zdir[opt_len] = '/';
            zdir[opt_len + 1] = '\0';
            retval = zdir;
        }
    }
    else { // relative path, or no path at all
        size_t cfg_dir_len = strlen(cfg_dir);

        if(opt) {
            size_t opt_len = strlen(opt);
            bool opt_has_slash = (opt[opt_len - 1] == '/');
            unsigned malloc_len = opt_len + cfg_dir_len + 1;
            if(!opt_has_slash) malloc_len++;
            char* zdir = malloc(malloc_len);
            char* zdptr = zdir;
            memcpy(zdptr, cfg_dir, cfg_dir_len);
            zdptr += cfg_dir_len;
            memcpy(zdptr, opt, opt_len);
            zdptr += opt_len;
            if(!opt_has_slash) *zdptr++ = '/';
            *zdptr = '\0';
            retval = zdir;
        }
        else {
            // no opt provided, just use cfg_dir
            retval = strdup(cfg_dir);
        }
    }

    struct stat zdstat;
    if(stat(retval, &zdstat))
        log_fatal("Cannot stat directory '%s': %s", retval, logf_errno());
    if((zdstat.st_mode & S_IFMT) != S_IFDIR)
        log_fatal("'%s' is not a directory", retval);

    return retval;
}

F_NONNULL
static bool bad_zattr(const char* key, unsigned klen V_UNUSED, const vscf_data_t* d V_UNUSED, void* zname) {
    assert(key); assert(zname);
    log_fatal("Invalid zone attribute key '%s' for zone '%s'", key, (const char*)zname);
}

// Makes a filename from a config directory and a name.
// (zones_dir + zone filename, curve_keys_dir + key filename, etc)
char* make_cf_fn(const char* dir, const char* fn) {
    assert(dir); assert(fn);

    if(fn[0] == '/')
        return strdup(fn);

    const unsigned dir_len = strlen(dir);
    const unsigned fn_len = strlen(fn);
    unsigned oal = dir_len + fn_len;

    char* retval = malloc(oal + 1);
    memcpy(retval, dir, dir_len);
    memcpy(retval + dir_len, fn, fn_len);
    retval[oal] = '\0';

    return retval;
}

F_NONNULL F_MALLOC F_WUNUSED
static uint8_t* make_zone_dname(const char* txtname, unsigned txtname_len) {
    assert(txtname);

    if(!txtname_len)
        log_fatal("Empty zone name in config (did you mean ., the root zone?)");

    uint8_t* dname = malloc(256);

    // XXX not only does this have double-escaping problems for dots embedded in labels...
    // It will probably crash on well-crafted invalid input in the config file, because
    //   no parser is pre-checking for escape sequence validity before this.  E.g.:
    //     zones => { example.co\\ => {} }
    // ... would decode to a string key with a trailing '\', which in turn will be crashy
    //   when dname_from_string() tries to decode that dangling escape...
    // The right fix is to upgrade vscf keys to be able to be native dnames in the same
    //   sense that simple string values are.  In practice that looks a little hairy to
    //   implement due to the special nature of keys (iteration callbacks, sorting, etc..).
    // For that matter all of the data interpretations on simple string values (e.g. integer)
    //   could be handy for keys in some cases...

    dname_status_t status = dname_from_string(dname, (const uint8_t*)txtname, txtname_len);
    if(status == DNAME_INVALID)
        log_fatal("Zone name '%s' is illegal", txtname);

    if(dname_iswild(dname))
        log_fatal("Zone '%s': Wildcard zone names not allowed", logf_dname(dname));

    if(status == DNAME_PARTIAL)
        dname_terminate(dname);

    return dname;
}

F_NONNULL
static bool configure_zone(const char* zname, unsigned znlen, const vscf_data_t* this_zone, void* data) {
    assert(zname); assert(this_zone); assert(data);

    if(!vscf_is_hash(this_zone))
        log_fatal("The value of zone '%s' in the configuration must be a hash", zname);

    if(!znlen)
        log_fatal("Empty zone names are not legal");

    const char* zones_dir = (const char*)data;

    const unsigned zidx = gconfig.num_zones++;

    {
        char* z = malloc(znlen + 1);
        memcpy(z, zname, znlen + 1);
        gconfig.zones[zidx].name = z;
    }
    gconfig.zones[zidx].dname = make_zone_dname(zname, znlen);
    gconfig.zones[zidx].zones_dir = strdup(zones_dir);

    const vscf_data_t* this_zone_file = vscf_hash_get_data_byconstkey(this_zone, "file", true);
    if(this_zone_file) {
        if(!vscf_is_simple(this_zone_file))
            log_fatal("Attribute 'file' for zone '%s' must have a string value", zname);
        gconfig.zones[zidx].file = make_cf_fn(zones_dir, vscf_simple_get_data(this_zone_file));
    }
    else {
        gconfig.zones[zidx].file = make_cf_fn(zones_dir, zname);
    }

    const vscf_data_t* this_zone_ttl = vscf_hash_get_data_byconstkey(this_zone, "default_ttl", true);
    if(this_zone_ttl) {
        unsigned long zttl;
        if(!vscf_is_simple(this_zone_ttl) || !vscf_simple_get_as_ulong(this_zone_ttl, &zttl))
            log_fatal("Attribute 'default_ttl' for zone '%s' must have a simple unsigned integer value", zname);
        if(zttl > 2147483647)
            log_fatal("Attribute 'default_ttl' for zone '%s' must be less than 2147483648", zname);
        gconfig.zones[zidx].def_ttl = zttl;
    }
    else {
        gconfig.zones[zidx].def_ttl = gconfig.zones_default_ttl;
    }

    vscf_hash_iterate(this_zone, true, bad_zattr, (void*)zname);
    return true;
}

F_NONNULL
static gen_func_ptr get_dlsym(void* handle, const char* plugin_name, const char* sym_suffix, bool required) {
    assert(handle); assert(plugin_name); assert(sym_suffix);

    char* symname;
    if(asprintf(&symname, "plugin_%s_%s", plugin_name, sym_suffix) < 0)
        log_fatal("asprintf() failed: %s", logf_errno());
    dlerror(); // clear previous errors
    gen_func_ptr retval = dlsym_fptr(handle, symname);
    const char* err = dlerror();
    if(required && (err || !retval)) log_fatal("Failed to resolve plugin symbol '%s': %s", symname, err);
    free(symname);
    return retval;
}

F_NONNULL
static void* dlopen_plugin(const char* name, const char** psearch) {
    assert(name); assert(psearch);

    char plugin_pathname[PATH_MAX];
    struct stat plugstat;
    const char* try_path;
    while((try_path = *psearch++)) {
        int written = snprintf(plugin_pathname, PATH_MAX, "%s/plugin_%s.so", try_path, name);
        if(written >= PATH_MAX) {
            log_warn("Could not construct plugin pathname for plugin '%s' at path '%s', names too long", name, try_path);
        }
        else {
            log_debug("Looking for plugin '%s' at pathname '%s'", name, plugin_pathname);
            if(0 == stat(plugin_pathname, &plugstat) && S_ISREG(plugstat.st_mode)) {
                void* plugin_handle = dlopen(plugin_pathname, RTLD_NOW);
                if(!plugin_handle)
                    log_fatal("Failed to dlopen() the '%s' plugin: %s", name, dlerror());
                return plugin_handle;
            }
        }
    }

    log_fatal("Failed to locate plugin '%s' in the plugin search path", name);
}

F_NONNULL
static bool load_plugin(const char* name, unsigned namelen V_UNUSED, const vscf_data_t* pconf, void* ps_asvoid) {
    assert(name); assert(pconf); assert(ps_asvoid);

    const char** psearch = (const char**) ps_asvoid;
    if(!vscf_is_hash(pconf))
        log_fatal("Config data for plugin '%s' must be a hash", name);

    log_debug("Assigning slot #%u to plugin '%s'", gconfig.num_plugins, name);
    gconfig.plugins = realloc(gconfig.plugins, (gconfig.num_plugins + 1) * sizeof(plugin_t));
    plugin_t* this_plugin = &gconfig.plugins[gconfig.num_plugins++];

    this_plugin->name = strdup(name);
    void* plugin_handle = dlopen_plugin(name, psearch);
    gdnsd_apiv_cb_t apiv = (gdnsd_apiv_cb_t)get_dlsym(plugin_handle, name, "get_api_version", true);
    unsigned this_version = apiv();
    if(this_version != GDNSD_PLUGIN_API_VERSION)
        log_fatal("Plugin '%s' needs to be recompiled (wanted API version %u, got %u)", name, GDNSD_PLUGIN_API_VERSION, this_version);

    gdnsd_load_config_cb_t load_config = (gdnsd_load_config_cb_t)get_dlsym(plugin_handle, this_plugin->name, "load_config", false);
    if(load_config) {
        monio_list_t* mon_list = load_config(pconf);
        if(mon_list) {
            for(unsigned i = 0; i < mon_list->count; i++) {
                monio_info_t* m = &mon_list->info[i];
                if(!m->desc)
                    log_fatal("Plugin '%s' bug: monio_info_t.desc is required", this_plugin->name);
                if(!m->addr)
                    log_fatal("Plugin '%s' bug: monio_info_t.addr is required", this_plugin->name);
                if(!m->state_ptr)
                    log_fatal("Plugin '%s' bug: monio_info_t.state_ptr is required", this_plugin->name);
                monio_add(m->svctype, m->desc, m->addr, m->state_ptr);
            }
        }
    }

    this_plugin->map_resource_dyna = (gdnsd_map_resource_dyna_cb_t)get_dlsym(plugin_handle, this_plugin->name, "map_resource_dyna", false);
    this_plugin->map_resource_dync = (gdnsd_map_resource_dync_cb_t)get_dlsym(plugin_handle, this_plugin->name, "map_resource_dync", false);
    this_plugin->full_config = (gdnsd_full_config_cb_t)get_dlsym(plugin_handle, this_plugin->name, "full_config", false);
    this_plugin->pre_privdrop = (gdnsd_pre_privdrop_cb_t)get_dlsym(plugin_handle, this_plugin->name, "pre_privdrop", false);
    this_plugin->pre_run = (gdnsd_pre_run_cb_t)get_dlsym(plugin_handle, this_plugin->name, "pre_run", false);
    this_plugin->iothread_init = (gdnsd_iothread_init_cb_t)get_dlsym(plugin_handle, this_plugin->name, "iothread_init", false);
    this_plugin->resolve_dynaddr = (gdnsd_resolve_dynaddr_cb_t)get_dlsym(plugin_handle, this_plugin->name, "resolve_dynaddr", false);
    this_plugin->resolve_dyncname = (gdnsd_resolve_dyncname_cb_t)get_dlsym(plugin_handle, this_plugin->name, "resolve_dyncname", false);
    this_plugin->exit = (gdnsd_exit_cb_t)get_dlsym(plugin_handle, this_plugin->name, "exit", false);

    return true;
}

// These defines are for the repetitive case of simple checking/assignment
//  of certain types directly into simple gconfig variables

#define CFG_OPT_BOOL(_opt_set, _gconf_loc) \
    do { \
        const vscf_data_t* _opt_setting = vscf_hash_get_data_byconstkey(_opt_set, #_gconf_loc, true); \
        if(_opt_setting) { \
            if(!vscf_is_simple(_opt_setting) \
            || !vscf_simple_get_as_bool(_opt_setting, &gconfig._gconf_loc)) \
                log_fatal("Config option %s: Value must be 'true' or 'false'", #_gconf_loc); \
        } \
    } while(0)

#define CFG_OPT_BOOL_ALTSTORE(_opt_set, _gconf_loc, _store) \
    do { \
        const vscf_data_t* _opt_setting = vscf_hash_get_data_byconstkey(_opt_set, #_gconf_loc, true); \
        if(_opt_setting) { \
            if(!vscf_is_simple(_opt_setting) \
            || !vscf_simple_get_as_bool(_opt_setting, &_store)) \
                log_fatal("Config option %s: Value must be 'true' or 'false'", #_gconf_loc); \
        } \
    } while(0)

#define CFG_OPT_UINT(_opt_set, _gconf_loc, _min, _max) \
    do { \
        const vscf_data_t* _opt_setting = vscf_hash_get_data_byconstkey(_opt_set, #_gconf_loc, true); \
        if(_opt_setting) { \
            unsigned long _val; \
            if(!vscf_is_simple(_opt_setting) \
            || !vscf_simple_get_as_ulong(_opt_setting, &_val)) \
                log_fatal("Config option %s: Value must be a positive integer", #_gconf_loc); \
            if(_val < _min || _val > _max) \
                log_fatal("Config option %s: Value out of range (%lu, %lu)", #_gconf_loc, _min, _max); \
            gconfig._gconf_loc = (unsigned) _val; \
        } \
    } while(0)

#define CFG_OPT_INT(_opt_set, _gconf_loc, _min, _max) \
    do { \
        const vscf_data_t* _opt_setting = vscf_hash_get_data_byconstkey(_opt_set, #_gconf_loc, true); \
        if(_opt_setting) { \
            long _val; \
            if(!vscf_is_simple(_opt_setting) \
            || !vscf_simple_get_as_long(_opt_setting, &_val)) \
                log_fatal("Config option %s: Value must be an integer", #_gconf_loc); \
            if(_val < _min || _val > _max) \
                log_fatal("Config option %s: Value out of range (%li, %li)", #_gconf_loc, _min, _max); \
            gconfig._gconf_loc = (int) _val; \
        } \
    } while(0)

#define CFG_OPT_UINT_ALTSTORE(_opt_set, _gconf_loc, _min, _max, _store) \
    do { \
        const vscf_data_t* _opt_setting = vscf_hash_get_data_byconstkey(_opt_set, #_gconf_loc, true); \
        if(_opt_setting) { \
            unsigned long _val; \
            if(!vscf_is_simple(_opt_setting) \
            || !vscf_simple_get_as_ulong(_opt_setting, &_val)) \
                log_fatal("Config option %s: Value must be a positive integer", #_gconf_loc); \
            if(_val < _min || _val > _max) \
                log_fatal("Config option %s: Value out of range (%lu, %lu)", #_gconf_loc, _min, _max); \
            _store = (unsigned) _val; \
        } \
    } while(0)

#define CFG_OPT_STR(_opt_set, _gconf_loc) \
    do { \
        const vscf_data_t* _opt_setting = vscf_hash_get_data_byconstkey(_opt_set, #_gconf_loc, true); \
        if(_opt_setting) { \
            if(!vscf_is_simple(_opt_setting)) \
                log_fatal("Config option %s: Wrong type (should be string)", #_gconf_loc); \
            gconfig._gconf_loc = strdup(vscf_simple_get_data(_opt_setting)); \
        } \
    } while(0)

#define CFG_OPT_STR_NOCOPY(_opt_set, _name, _store_at) \
    do { \
        const vscf_data_t* _opt_setting = vscf_hash_get_data_byconstkey(_opt_set, _name, true); \
        if(_opt_setting) { \
            if(!vscf_is_simple(_opt_setting)) \
                log_fatal("Config option %s: Wrong type (should be string)", _name); \
            _store_at = vscf_simple_get_data(_opt_setting); \
        } \
    } while(0)

F_NONNULL
static void add_subzone(zoneinfo_t* z, const uint8_t* child) {
    assert(z); assert(child);
    log_debug("Adding '%s' as a child zone of zone '%s'", logf_dname(child), logf_dname(z->dname));
    z->subzones = realloc(z->subzones, sizeof(uint8_t*) * (z->n_subzones + 1));
    z->subzones[z->n_subzones++] = child;
}

// Sort the zones array by the zone name length descending
F_PURE F_NONNULL
static int zones_cmp(const zoneinfo_t* a, const zoneinfo_t* b) {
    assert(a); assert(b);
    assert(a->dname); assert(b->dname);
    return dname_cmp(a->dname, b->dname);
}

// This operates on gconfig.zones, accomplishing two primary tasks:
//  (1) Catching duplicate zones with a fatal error
//  (2) Setting up the subzones array for each zone, which is a list
//    of explicitly-loaded subzones of a given zone.
static void postproc_zones(void) {
    // Because these will be frequently referenced
    zoneinfo_t* zones = gconfig.zones;
    const unsigned num_zones = gconfig.num_zones;

    // Sort zones by zone name length ascending.
    // Note that with dname_cmp as the comparator,
    //  any duplicates would be paired in the list
    qsort(zones, num_zones, sizeof(zoneinfo_t),
        (int (*)(const void*, const void*))zones_cmp);

    // Walk down the list from longest to shortest zone name
    for(int zidx1 = num_zones - 1; zidx1 > 0; zidx1--) {

        // Check next entry for a duplicate
        const int zidx_next = zidx1 - 1;
        if(!dname_cmp(zones[zidx_next].dname, zones[zidx1].dname))
            log_fatal("Zone name duplicated in config: '%s'", logf_dname(zones[zidx1].dname));

        // Walk down remainder of list from longest to shortest zone name
        for(int zidx2 = zidx_next; zidx2 > -1; zidx2--) {
            if(dname_isparentof(zones[zidx2].dname, zones[zidx1].dname)) {
                add_subzone(&zones[zidx2], zones[zidx1].dname);
                break; // a zone can only have one parent zone
            }
        }
    }
}

static void process_http_listen(const vscf_data_t* http_listen_opt) {
    if(!http_listen_opt || !vscf_array_get_len(http_listen_opt)) {
        gconfig.num_http_addrs = 1;
#if USE_V6SOCK
        const bool v6works = udp_sock_v6test();
        if(v6works)
            gconfig.num_http_addrs = 2;
        else
            log_info("Runtime IPv6 support missing, disabling default v6 HTTP listener");
#endif
        gconfig.http_addrs = calloc(gconfig.num_http_addrs, sizeof(anysin_t));
        make_addr("0.0.0.0", gconfig.http_addrs, 3506);
#if USE_V6SOCK
        if(v6works)
            make_addr("::", &gconfig.http_addrs[1], 3506);
#endif
    }
    else {
        gconfig.num_http_addrs = vscf_array_get_len(http_listen_opt);
        gconfig.http_addrs = calloc(gconfig.num_http_addrs, sizeof(anysin_t));
        for(unsigned i = 0; i < gconfig.num_http_addrs; i++) {
            const vscf_data_t* lspec = vscf_array_get_data(http_listen_opt, i);
            if(!vscf_is_simple(lspec))
                log_fatal("Config option 'http_listen': all listen specs must be strings");
            make_addr(vscf_simple_get_data(lspec), &gconfig.http_addrs[i], 3506);
        }
    }
}

F_NONNULLX(1)
static void set_dnsaddr_keys(dns_addr_t* addrconf, const vscf_data_t* addr_keylist) {
    assert(addrconf);
    if(!addr_keylist) { // no per-address curve_keys array, use all
        unsigned n_keys = gconfig.num_curve_keys;
        if(n_keys) {
            addrconf->num_curve_keys = n_keys;
            addrconf->curve_keys = calloc(n_keys, sizeof(dnsaddr_key_t));
            for(unsigned i = 0; i < n_keys; i++) {
                curvekey_t* this_key = &gconfig.curve_keys[i];
                addrconf->curve_keys[i].key = this_key;
                this_key->num_socks += (addrconf->tcp_disabled ? 1 : 2);
            }
        }
    }
    else {
        unsigned n_local_keys = vscf_array_get_len(addr_keylist);
        if(n_local_keys) {
            unsigned n_global_keys = gconfig.num_curve_keys;
            addrconf->num_curve_keys = n_local_keys;
            addrconf->curve_keys = calloc(n_local_keys, sizeof(dnsaddr_key_t));
            for(unsigned i = 0; i < n_local_keys; i++) {
                const vscf_data_t* kname_cfg = vscf_array_get_data(addr_keylist, i);
                if(!vscf_is_simple(kname_cfg))
                    log_fatal("per-address curve_keys value must be a single key name or an array of key names");
                const char* kname = vscf_simple_get_data(kname_cfg);
                for(unsigned j = 0; i < n_global_keys; j++) {
                    curvekey_t* this_key = &gconfig.curve_keys[j];
                    if(!strcmp(this_key->name, kname)) {
                        addrconf->curve_keys[i].key = this_key;
                        this_key->num_socks += (addrconf->tcp_disabled ? 1 : 2);
                        break;
                    }
                }
                if(!addrconf->curve_keys[i].key)
                    log_fatal("per-address curve_key name '%s' does not exist in global curve_keys config", kname);
            }
        }
    }
}

static void process_listen(const vscf_data_t* listen_opt, const unsigned def_tcp_cps, const unsigned def_tcp_to, const bool def_tcp_disabled, const unsigned def_udp_kc_bits, const unsigned def_tcp_kc_bits, const unsigned def_udp_recv_width, const unsigned def_udp_rcvbuf, const unsigned def_udp_sndbuf) {
    if(!listen_opt || !vscf_array_get_len(listen_opt)) {
        gconfig.num_dns_addrs = 1;
#if USE_V6SOCK
        const bool v6works = udp_sock_v6test();
        if(v6works)
            gconfig.num_dns_addrs = 2;
        else
            log_info("Runtime IPv6 support missing, disabling default v6 DNS listener");
#endif
        gconfig.dns_addrs = calloc(gconfig.num_dns_addrs, sizeof(dns_addr_t));
        dns_addr_t* addrconf = &gconfig.dns_addrs[0];
        make_addr("0.0.0.0", &addrconf->addr, 53);
        addrconf->tcp_clients_per_socket = def_tcp_cps;
        addrconf->tcp_timeout = def_tcp_to;
        addrconf->udp_kc_bits = def_udp_kc_bits;
        addrconf->tcp_kc_bits = def_tcp_kc_bits;
        addrconf->tcp_disabled = def_tcp_disabled;
        addrconf->udp_recv_width = def_udp_recv_width;
        addrconf->udp_rcvbuf = def_udp_rcvbuf;
        addrconf->udp_sndbuf = def_udp_sndbuf;
        set_dnsaddr_keys(addrconf, NULL);

#if USE_V6SOCK
        if(v6works) {
            addrconf = &gconfig.dns_addrs[1];
            make_addr("::", &addrconf->addr, 53);
            addrconf->tcp_clients_per_socket = def_tcp_cps;
            addrconf->tcp_timeout = def_tcp_to;
            addrconf->udp_kc_bits = def_udp_kc_bits;
            addrconf->tcp_kc_bits = def_tcp_kc_bits;
            addrconf->tcp_disabled = def_tcp_disabled;
            addrconf->udp_recv_width = def_udp_recv_width;
            addrconf->udp_rcvbuf = def_udp_rcvbuf;
            addrconf->udp_sndbuf = def_udp_sndbuf;
            set_dnsaddr_keys(addrconf, NULL);
        }
#endif
    }
    else {
        if(vscf_is_hash(listen_opt)) {
            gconfig.num_dns_addrs = vscf_hash_get_len(listen_opt);
            gconfig.dns_addrs = calloc(gconfig.num_dns_addrs, sizeof(dns_addr_t));
            for(unsigned i = 0; i < gconfig.num_dns_addrs; i++) {
                dns_addr_t* addrconf = &gconfig.dns_addrs[i];
                addrconf->tcp_clients_per_socket = def_tcp_cps;
                addrconf->tcp_timeout = def_tcp_to;
                addrconf->udp_kc_bits = def_udp_kc_bits;
                addrconf->tcp_kc_bits = def_tcp_kc_bits;
                addrconf->tcp_disabled = def_tcp_disabled;
                addrconf->udp_recv_width = def_udp_recv_width;
                addrconf->udp_rcvbuf = def_udp_rcvbuf;
                addrconf->udp_sndbuf = def_udp_sndbuf;
                const char* lspec = vscf_hash_get_key_byindex(listen_opt, i, NULL);
                make_addr(lspec, &addrconf->addr, 53);
                const vscf_data_t* addr_opts = vscf_hash_get_data_byindex(listen_opt, i);
                if(!vscf_is_hash(addr_opts))
                    log_fatal("per-address options for listen address '%s' must be a hash", lspec);

                CFG_OPT_UINT_ALTSTORE(addr_opts, tcp_clients_per_socket, 1LU, 65535LU, addrconf->tcp_clients_per_socket);
                CFG_OPT_UINT_ALTSTORE(addr_opts, tcp_timeout, 3LU, 60LU, addrconf->tcp_timeout);
                CFG_OPT_UINT_ALTSTORE(addr_opts, udp_keycache_bits, 6LU, 32LU, addrconf->udp_kc_bits);
                CFG_OPT_UINT_ALTSTORE(addr_opts, tcp_keycache_bits, 6LU, 32LU, addrconf->tcp_kc_bits);
                CFG_OPT_BOOL_ALTSTORE(addr_opts, disable_tcp, addrconf->tcp_disabled);
                CFG_OPT_UINT_ALTSTORE(addr_opts, udp_recv_width, 1LU, 32LU, addrconf->udp_recv_width);
                CFG_OPT_UINT_ALTSTORE(addr_opts, udp_rcvbuf, 4096LU, 1048576LU, addrconf->udp_rcvbuf);
                CFG_OPT_UINT_ALTSTORE(addr_opts, udp_sndbuf, 4096LU, 1048576LU, addrconf->udp_sndbuf);
                set_dnsaddr_keys(addrconf, vscf_hash_get_data_byconstkey(addr_opts, "curve_keys", true));
                vscf_hash_iterate(addr_opts, true, bad_key, "per-address listen option");
            }
        }
        else {
            gconfig.num_dns_addrs = vscf_array_get_len(listen_opt);
            gconfig.dns_addrs = calloc(gconfig.num_dns_addrs, sizeof(dns_addr_t));
            for(unsigned i = 0; i < gconfig.num_dns_addrs; i++) {
                dns_addr_t* addrconf = &gconfig.dns_addrs[i];
                addrconf->tcp_clients_per_socket = def_tcp_cps;
                addrconf->tcp_timeout = def_tcp_to;
                addrconf->udp_kc_bits = def_udp_kc_bits;
                addrconf->tcp_kc_bits = def_tcp_kc_bits;
                addrconf->tcp_disabled = def_tcp_disabled;
                addrconf->udp_recv_width = def_udp_recv_width;
                addrconf->udp_rcvbuf = def_udp_rcvbuf;
                addrconf->udp_sndbuf = def_udp_sndbuf;
                const vscf_data_t* lspec = vscf_array_get_data(listen_opt, i);
                if(!vscf_is_simple(lspec))
                    log_fatal("Config option 'listen': all listen specs must be strings");
                make_addr(vscf_simple_get_data(lspec), &addrconf->addr, 53);
                set_dnsaddr_keys(addrconf, NULL);
            }
        }
    }
}

static void assign_thread_nums(void) {
    unsigned tnum = 0;
    unsigned addr_ct = gconfig.num_dns_addrs;

    for(unsigned i = 0; i < addr_ct; i++)
        gconfig.dns_addrs[i].udp_threadnum = tnum++;

    for(unsigned i = 0; i < addr_ct; i++)
        if(!gconfig.dns_addrs[i].tcp_disabled)
            gconfig.dns_addrs[i].tcp_threadnum = tnum++;

    gconfig.num_io_threads = tnum;
}

void conf_load(const char* cfg_file) {
    assert(cfg_file);
    log_debug("Loading configuration");

    char* vscf_err;
    const vscf_data_t* cfg_root = vscf_scan_filename(cfg_file, &vscf_err);
    if(!cfg_root)
        log_fatal("Configuration load failed: %s", vscf_err);

    assert(vscf_is_hash(cfg_root));

    // cfg_dir will have a /-terminated directory
    //  name (cleaned up wrt symlinks and /../) for
    //  where the config file was located.  Needs
    //  to be free()'d later.
    char* cfg_dir;
    {
        char* real_config_pathname = absify(cfg_file);
        char* tmp_cfg_dir = dirname(real_config_pathname);
        if(!tmp_cfg_dir) log_fatal("dirname(%s) failed: %s", real_config_pathname, logf_errno());

        unsigned tmp_cfg_dir_len = strlen(tmp_cfg_dir);
        cfg_dir = malloc(tmp_cfg_dir_len + 2);
        memcpy(cfg_dir, tmp_cfg_dir, tmp_cfg_dir_len);
        cfg_dir[tmp_cfg_dir_len] = '/';
        cfg_dir[tmp_cfg_dir_len + 1] = '\0';
        free(real_config_pathname);
    }

    const vscf_data_t* options = vscf_hash_get_data_byconstkey(cfg_root, "options", true);

    const char* zdopt = NULL;
    const char* kdopt = NULL;
    char* zones_dir = NULL;
    char* keys_dir = NULL;
    const vscf_data_t* listen_opt = NULL;
    const vscf_data_t* http_listen_opt = NULL;
    const vscf_data_t* psearch_array = NULL;
    unsigned def_tcp_cps = 128U;
    unsigned def_tcp_to = 5U;
    unsigned def_udp_kc_bits = 9U;
    unsigned def_tcp_kc_bits = 7U;
    unsigned def_udp_recv_width = 4U;
    unsigned def_udp_rcvbuf = 0U;
    unsigned def_udp_sndbuf = 0U;
    bool def_tcp_disabled = false;

    if(options) {
        if(!vscf_is_hash(options))
            log_fatal("Config key 'options': wrong type (must be hash)");
        CFG_OPT_BOOL_ALTSTORE(options, debug, dmn_debug);
        CFG_OPT_INT(options, priority, -20L, 20L);
        CFG_OPT_BOOL(options, include_optional_ns);
        CFG_OPT_BOOL(options, realtime_stats);
        CFG_OPT_BOOL(options, lock_mem);
        CFG_OPT_BOOL(options, disable_text_autosplit);
        CFG_OPT_BOOL(options, strict_data);
        CFG_OPT_UINT(options, log_stats, 1LU, 2147483647LU);
        CFG_OPT_UINT(options, max_http_clients, 1LU, 65535LU);
        CFG_OPT_UINT(options, http_timeout, 3LU, 60LU);
        CFG_OPT_UINT_ALTSTORE(options, tcp_clients_per_socket, 1LU, 65535LU, def_tcp_cps);
        CFG_OPT_UINT_ALTSTORE(options, tcp_timeout, 3LU, 60LU, def_tcp_to);
        CFG_OPT_UINT_ALTSTORE(options, udp_keycache_bits, 6LU, 32LU, def_udp_kc_bits);
        CFG_OPT_UINT_ALTSTORE(options, tcp_keycache_bits, 6LU, 32LU, def_tcp_kc_bits);
        CFG_OPT_BOOL_ALTSTORE(options, disable_tcp, def_tcp_disabled);
        CFG_OPT_UINT_ALTSTORE(options, udp_recv_width, 1LU, 32LU, def_udp_recv_width);
        CFG_OPT_UINT_ALTSTORE(options, udp_rcvbuf, 4096LU, 1048576LU, def_udp_rcvbuf);
        CFG_OPT_UINT_ALTSTORE(options, udp_sndbuf, 4096LU, 1048576LU, def_udp_sndbuf);
        CFG_OPT_UINT(options, zones_default_ttl, 1LU, 2147483647LU);
        // 61K rather than 64K, to leave room for headers and DNSCurve TXT stuff...
        CFG_OPT_UINT(options, max_response, 4096LU, 62464LU);
        CFG_OPT_UINT(options, max_cname_depth, 4LU, 64LU);
        CFG_OPT_STR(options, pidfile);
        CFG_OPT_STR(options, username);
        CFG_OPT_STR(options, chroot_path);
        CFG_OPT_STR_NOCOPY(options, "zones_dir", zdopt);
        CFG_OPT_STR_NOCOPY(options, "curve_keys_dir", kdopt);
        listen_opt = vscf_hash_get_data_byconstkey(options, "listen", true);
        http_listen_opt = vscf_hash_get_data_byconstkey(options, "http_listen", true);
        psearch_array = vscf_hash_get_data_byconstkey(options, "plugin_search_path", true);
        vscf_hash_iterate(options, true, bad_key, "options");
    }

    // In the non-DNSCurve case these are identical...
    gconfig.outer_max_response = gconfig.max_response;

    // Potentially subdirectories of cfg_dir
    zones_dir = configure_cfdir(cfg_dir, zdopt);
#if DNSCURVE
    keys_dir = configure_cfdir(cfg_dir, kdopt);
#else
    if(kdopt)
        log_fatal("This server was not built with DNSCurve support, and thus keys_dir is illegal");
#endif
    free(cfg_dir);

    // Set up the http listener data
    process_http_listen(http_listen_opt);

    const vscf_data_t* curve_keys_cfg = vscf_hash_get_data_byconstkey(cfg_root, "curve_keys", true);
#if DNSCURVE
    // Initial setup of the global gconfig.curve_keys data
    make_key_data(keys_dir, curve_keys_cfg);
#else
    if(curve_keys_cfg)
        log_fatal("This server was not built with DNSCurve support, and thus curve_keys is illegal");
#endif

    free(keys_dir);

#if DNSCURVE
    // Size the outer response buffer for DNSCurve if applicable
    // Slightly pessimistic, but reasonable.
    if(gconfig.num_curve_keys) {
        // inner packet is box-inflated by 16 bytes, combined w/ 96-bit noncex,
        // and then chunked into 255-byte TXT parts, adding a length byte for each chunk.
        // TXT answer record has 0xC00C, class/type, TTL, and rdatalen as well.
        unsigned extra = ((gconfig.max_response + 16 + 12) / 255) + 1 + 12;
        // This is all tacked onto the TXT-format outer query packet.  I think
        //  the precise max for that is 1209 in the case of a full 512-byte inner
        //  query packet, but I've rounded up to 1280 just in case.
        gconfig.outer_max_response = 1280 + gconfig.max_response + extra;
    }
#endif

    // Initial setup of the listener data, makes use of gconfig.curve_keys, modding
    //  the per-key num_socks as it goes and referencing them in the dnsaddr_t's
    process_listen(listen_opt, def_tcp_cps, def_tcp_to, def_tcp_disabled, def_udp_kc_bits, def_tcp_kc_bits, def_udp_recv_width, def_udp_rcvbuf, def_udp_sndbuf);

#if DNSCURVE
    // Calculate the per-key sep_mask (iterates gconfig.curve_keys, uses the initial
    //  config values from make_key_data() combined with the num_socks values set
    //  above in process_listen()).
    calc_sep_masks();

    // Calculate the per-socket sep_val (iterates the dns_addrs, follows references
    //   into gconfig.curve_keys, uses the previously calculated num_socks and sep_mask
    //   values, also maintains and uses cur_sock to know the final local value).
    calc_sep_vals();
#endif

    // Assign globally unique thread numbers for each socket-handling thread
    assign_thread_nums();

    monio_add_servicetypes(vscf_hash_get_data_byconstkey(cfg_root, "service_types", true));

    const vscf_data_t* zones = vscf_hash_get_data_byconstkey(cfg_root, "zones", true);
    if(!zones)
        log_fatal("No zones specified");
    if(!vscf_is_hash(zones))
        log_fatal("Zones value is wrong type (must be hash)");

    unsigned n_zones = vscf_hash_get_len(zones);
    if(!n_zones)
        log_fatal("No zones specified");

    // This creates the gconfig.zones array.
    //   configure_zone increments gconfig.num_zones as it goes.
    gconfig.zones = calloc(sizeof(zoneinfo_t), n_zones);
    vscf_hash_iterate(zones, false, configure_zone, (void*)zones_dir);
    assert(gconfig.num_zones == n_zones);
    free(zones_dir);

    postproc_zones();

    int psearch_count = psearch_array
        ? vscf_array_get_len(psearch_array)
        : 0;
    const char** psearch = malloc((psearch_count + 2) * sizeof(const char*));
    for(int i = 0; i < psearch_count; i++) {
        const vscf_data_t* psd = vscf_array_get_data(psearch_array, i);
        if(!vscf_is_simple(psd))
            log_fatal("Plugin search paths must be strings");
        psearch[i] = vscf_simple_get_data(psd);
    }

    psearch[psearch_count++] = LIBDIR "/" PACKAGE_NAME;
    psearch[psearch_count] = NULL;

    // Load plugins
    const vscf_data_t* plugins_hash = vscf_hash_get_data_byconstkey(cfg_root, "plugins", true);
    if(plugins_hash) {
        if(!vscf_is_hash(plugins_hash))
            log_fatal("Config setting 'plugins' must have a hash value");
        vscf_hash_iterate(plugins_hash, false, load_plugin, (void*)psearch);
    }

    free(psearch);

    // Throw an error if there are any other unretrieved root config keys
    vscf_hash_iterate(cfg_root, true, bad_key, "top-level config");
    vscf_destroy(cfg_root);
}

void dns_lsock_init(void) {
    const unsigned num_addrs = gconfig.num_dns_addrs;
    for(unsigned i = 0; i < num_addrs; i++) {
        dns_addr_t* addrconf = &gconfig.dns_addrs[i];
        udp_sock_setup(addrconf);
        if(!addrconf->tcp_disabled)
            addrconf->tcp_sock = tcp_listen_setup(&addrconf->addr, addrconf->tcp_timeout, addrconf->tcp_clients_per_socket);
    }
}
