
#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <assert.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/mman.h>

#include <crypto_box_curve25519xsalsa20poly1305.h>
#include <crypto_scalarmult_curve25519.h>

#define fatal(msg, ...) do {\
    fprintf(stderr, msg "\n", __VA_ARGS__);\
    abort();\
} while(0)

#define warn(msg, ...) do {\
    fprintf(stderr, msg "\n", __VA_ARGS__);\
} while(0)

#define KEYLEN     32
#define KEYLEN_B32 51
#define KEYLEN_S   "32"

static unsigned char pk[KEYLEN] = { 0 };
static unsigned char* sk;
#define WIPE_SECRET memset(sk, 0, KEYLEN)

// This base32 code is based on agl's python implementation from
//  http://github.com/agl/dnscurve/blob/master/tools/base32.py
static const char b32_alphabet[] = "0123456789bcdfghjklmnpqrstuvwxyz";
static void display_public_key(void) {
   char out[KEYLEN_B32 + 1];
   unsigned out_idx = 0;
   unsigned v = 0;
   unsigned vbits = 0;
   for(unsigned i = 0; i < KEYLEN; i++) {
        v |= pk[i] << vbits;
        vbits += 8;
        while(vbits >= 5) {
             out[out_idx++] = b32_alphabet[v & 31];
             v >>= 5;
             vbits -= 5;
        }
   }
   assert(out_idx == KEYLEN_B32);
   out[KEYLEN_B32] = 0;

   printf("Public key as a DNS label: uz5%s\n", out);
}

static void make_new_key(const char* fn) {
    int fd = open(fn, O_WRONLY|O_CREAT|O_EXCL|O_NOFOLLOW, S_IRUSR);
    if(fd < 0)
        fatal("Error creating '%s': %s", fn, strerror(errno));

    // If /dev/urandom doesn't exist, the current NaCl
    //  randombytes() will sleep indefinitely here.  I suspect
    //  this is one of many reasons why it's not in the main
    //  library yet (another being that /dev/urandom isn't quite
    //  universal).  randombytes() is used internally by _keypair()
    crypto_box_curve25519xsalsa20poly1305_keypair(pk, sk);
    ssize_t writerv = write(fd, sk, KEYLEN);
    WIPE_SECRET;

    if(writerv != KEYLEN) {
        unlink(fn);
        if(writerv < 0)
            fatal("Write of secret key data to '%s' failed: %s", fn, strerror(errno));
        else
            fatal("Write of secret key data to '%s' failed: short write", fn);
    }

    if(close(fd)) {
        unlink(fn);
        fatal("Closing secret key file '%s' failed: %s", fn, strerror(errno));
    }

    printf("Created new secret key as '%s'\n", fn);
}

static void read_old_key(const char* fn) {
    int fd = open(fn, O_RDONLY|O_NOFOLLOW, S_IRUSR);
    if(fd < 0)
        fatal("Error opening '%s' for reading: %s", fn, strerror(errno));

    ssize_t readrv = read(fd, sk, KEYLEN);
    if(readrv != KEYLEN) {
        WIPE_SECRET;
        if(readrv < 0)
            fatal("Read of secret key data from '%s' failed: %s", fn, strerror(errno));
        else
            fatal("Read of secret key data from '%s' failed: short read", fn);
    }
    crypto_scalarmult_curve25519_base(pk,sk);
    WIPE_SECRET;

    if(close(fd))
        fatal("Closing secret key file '%s' failed: %s", fn, strerror(errno));

    printf("Using existing secret key from '%s'\n", fn);
}

#ifndef MAP_ANONYMOUS
#define MAP_ANONYMOUS MAP_ANON
#endif

int main(int argc, char* argv[]) {
    if(argc != 2)
        fatal("Usage: %s secret-key-file\n"
              "  If secret-key-file does not exist, it will be created as a new secret key\n"
              "  Regardless, the corresponding public key will be displayed\n", argv[0]);

    const char* fn = argv[1];
    struct stat fnstat;

    sk = mmap(NULL, KEYLEN, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS, -1, 0);
    if(!sk || mlock(sk, KEYLEN))
        fatal("Failed to mmap and mlock a region of size " KEYLEN_S ": %s", strerror(errno));
    WIPE_SECRET;

    if(lstat(fn, &fnstat)) {
        if(errno == ENOENT) {
            make_new_key(fn);
        }
        else {
            fatal("lstat('%s') failed: %s", fn, strerror(errno));
        }
    }
    else {
        if((fnstat.st_mode & S_IFMT) != S_IFREG)
            fatal("Existing file '%s' is not a regular file", fn);
        if((fnstat.st_mode & 0777) != 0400)
            warn("Existing keyfile '%s' has lax permissions (should be 0400)", fn);
        if(fnstat.st_size != 32)
            fatal("Existing file '%s' is not exactly 32 bytes long", fn);

        read_old_key(fn);
    }

    display_public_key();
    return 0;
}

