/* Copyright © 2010 Brandon L Black <blblack@gmail.com>
 *
 * This file is part of gdnsd.
 *
 * gdnsd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * gdnsd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with gdnsd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "dnscurve.h"

#include <sys/mman.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <crypto_box_curve25519xsalsa20poly1305.h>
#include <crypto_core_salsa208.h>
#include <randombytes.h>
#include <time.h>

#include "conf.h"
#include "dnscurve_data.h"

#ifndef MAP_ANONYMOUS
#define MAP_ANONYMOUS MAP_ANON
#endif

// The nonce64_state_t type is over in dnspacket.h.
// This is our 64-bit nonce generator, using the output
//   of salsa20/8 (64 bits at a time) to xor-encrypt
//   a 64-bit packet counter.  salsa20/8 is initialized
//   with a new random key and nonce each time a thread
//   starts, and the counter is initialized to a random
//   value as well.  Since salsa20/8 provides 512 bits
//   of output per invocation, we buffer it and consume
//   64 bits per generated nonce (so salsa gets invoked
//   to refill the buffer once per 8 nonce requests).

// DNSCurve actually uses a 96-bit server nonce extension,
//   so we still have 32 bits to fill.  Frankly I suspect
//   our decent-quality 64-bit nonce is "enough", and we
//   also need to remember to reserve bits for nonce
//   seperation (both between external servers and
//   between internal threads that share little state).
// For now, I'm putting the 32-bit time_t value there,
//   and barring coming up with any better plan, we'll be
//   doing nonce seperation by overwriting the highest
//   bits of that.

uint8_t* rpool;
static const unsigned char sigma[16] = "expand 32-byte k";

static void init_nonce64(dnspacket_context_t* c, const uint8_t* rbytes) {
    nonce64_state_t* s = c->n64_state
        = malloc(sizeof(nonce64_state_t));

    memcpy(s->in, rbytes, 8);
    memcpy(&s->c, rbytes + 8, 8);
    memcpy(s->k, rbytes + 16, 32);
    memset(s->in + 8, 0, 8);
    s->nbuf_remain = 0;
}

static uint64_t next_nonce64(nonce64_state_t* s) {
    int32_t  i;
    uint32_t u;

    if(!s->nbuf_remain) {
        s->nbuf_remain = 8;
        crypto_core_salsa208((uint8_t*)s->nbuf,s->in,s->k,sigma);
        u = 1;
        for (i = 8;i < 16;++i) {
            u += (uint32_t) s->in[i];
            s->in[i] = u;
            u >>= 8;
        }
    }

    return s->nbuf[(8 - s->nbuf_remain--)] ^ s->c++;
}

// This pulls the good 64-bit nonce from next_nonce64()
//  and builds a complete 96-bit nonce extension by
//  prepending 32-bit time_t and applying any neccessary
//  nonce separation bits at the top.
// If the 64-bit PRNG (which steps per packet) were to cycle,
//  its cycle would have to coincide with a cycle in the
//  32 upper bits in order to generate a previously-used nonce.
// Even if nonce separation steals 8 bits (which means globally
//  there are 256 listening sockets on the internet somewhere
//  knowingly sharing the same private key - I doubt that number
//  will be seen often in practice), the cycle time on those
//  upper 32 bits would be ~194 days.
static void noncex96(nonce64_state_t* s, uint32_t sep_mask, uint32_t sep_val, uint8_t* nxout) {
    uint32_t t = htonl((((uint32_t)time(NULL)) & sep_mask) | sep_val);
    uint64_t nx64 = next_nonce64(s);
    memcpy(nxout, &t, 4);
    memcpy(nxout + 4, &nx64, 8);
}

// Not really "secure", but it is a locked fresh mmap() region.  This prevents
//  writing it to swap, and reduces the odds that a bug elsewhere in the code
//  will accidentally divulge keys, since the keys are in a distinct mapping
//  (page address) that isn't shared with other data.
F_MALLOC F_WUNUSED
static void* alloc_secmem(unsigned size) {
    void* rv = mmap(NULL, size, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS, -1, 0);
    if(!gconfig.lock_mem && mlock(rv, size))
        log_fatal("Could not mlock() a region of size %u for secret data: %s", size, logf_errno());
    memset(rv, 0, size);
    return rv;
}

static unsigned global_num_curve_keys = 0;
static uint8_t* global_curve_secrets = NULL;

static void destroy_curve_key_data(void) {
    // log_debug("Destroying global secret data in main thread");
    memset(global_curve_secrets, 0, 32 * global_num_curve_keys);
}

void load_curve_key_data(void) {
    if(!(global_num_curve_keys = gconfig.num_curve_keys))
        return;

    global_curve_secrets = alloc_secmem(32 * global_num_curve_keys);
    if(atexit(destroy_curve_key_data))
        log_fatal("atexit() failed: %s", logf_errno());

    for(unsigned i = 0; i < global_num_curve_keys; i++) {
        const char* fn = gconfig.curve_keys[i].file;
        uint8_t* secret = global_curve_secrets + (i * 32);
        gconfig.curve_keys[i].secret = secret;
        int fd = open(fn, O_RDONLY|O_NOFOLLOW);
        if(fd < 0)
            log_fatal("Could not open secret key file '%s': %s", fn, logf_errno());
        ssize_t readrv = read(fd, secret, 32);
        if(readrv != 32) {
            if(readrv < 0)
                log_fatal("read(%s) failed: %s", fn, logf_errno());
            else
                log_fatal("read(%s) failed: short read", fn);
        }
        close(fd);
    }
}

F_NONNULL
static void load_thread_keys(dnspacket_context_t* pctx, const dns_addr_t* addrconf) {
    unsigned num_curve_keys = pctx->num_curve_keys = addrconf->num_curve_keys;
    assert(num_curve_keys);

    io_curvekey_t* curve_keys = pctx->curve_keys
        = alloc_secmem(sizeof(io_curvekey_t) * num_curve_keys);

    for(unsigned i = 0; i < num_curve_keys; i++) {
        memcpy(&curve_keys[i].key, addrconf->curve_keys[i].key->secret, 32);
        curve_keys[i].sep_mask = addrconf->curve_keys[i].key->sep_mask;
        curve_keys[i].sep_val  = pctx->is_udp
            ? addrconf->curve_keys[i].udp_sep_val
            : addrconf->curve_keys[i].tcp_sep_val;
    }
}

void destroy_thread_curve_keys(dnspacket_context_t* pctx) {
    assert(pctx);
    // log_debug("Destroying per-thread secret data in thread %u", pctx->threadnum);
    if(pctx->curve_keys)
        memset(pctx->curve_keys, 0, pctx->num_curve_keys * sizeof(io_curvekey_t));
    if(pctx->keycache)
        memset(pctx->keycache->data_array, 0, pctx->keycache->size * pctx->keycache->data_size);
}

/*
 The key cache:

 One of the design constraints here is no dynamic memory allocation
  at runtime.
 The current design is basically an array of cache elements, a hash index
  for lookups, and a CLOCK-like algorithm for evictions.  Instead of having
  a single reference bit, we increment a 32-bit ref count on every cache
  hit.  As the clock hand sweeps, we evict the first element with zero
  reference count as usual.  All non-zero ref counts we encounter as
  the hand sweeps are shifted right by one bit, halving their values.
 The theoretical worst-case here is 32 full sweeps to find an eviction target,
  but in practice it would be difficult for such scenarios to ever arise.
 The array elements have something like a doubly-linked-list structure for
  hash collision chaining, which also allows for trivial removal of evicted
  cache items from the hash index.
 The actual hash algorithm is Jenkins' lookup3 (which has 96-bit
  internal state and is known to be more resistant than other simple
  hashes to collision-based performance attacks), and has a random
  (generated at startup) 32-bit salt/init value.
*/

F_NONNULL
static void kc_init(dnspacket_context_t* c, const dns_addr_t* addrconf, uint32_t salt) {
    assert(c); assert(addrconf);

    const unsigned kc_bits = c->is_udp ? addrconf->udp_kc_bits : addrconf->tcp_kc_bits;
    assert(kc_bits > 5); assert(kc_bits < 33);

    const unsigned n_keys = c->num_curve_keys;
    const unsigned data_size = sizeof(kc_data_t) + (32 * n_keys);
    const unsigned array_size = 1U << kc_bits;
    kc_t* kc = malloc(sizeof(kc_t));
    kc->hand = 0;
    kc->salt = salt;
    kc->size = array_size;
    kc->data_size = data_size;
    kc->cache_hit = &c->stats->curve_cache_hit;
    kc->cache_miss = &c->stats->curve_cache_miss;
    kc->data_array = alloc_secmem(array_size * data_size);
    kc->hash = calloc(array_size << 1U, sizeof(kc_data_t*));
    c->keycache = kc;
}

#include "lookup3_pk.h"

// The cache entry fetcher, which handles insertion/eviction as we go
F_NONNULL
static kc_data_t* kc_fetch(kc_t* kc, const uint8_t* pubkey) {
    assert(kc); assert(pubkey);

    const uint32_t hashmask = (kc->size << 1U) - 1U;
    const uint32_t hashval = pubkey_lookup3((const uint32_t*)pubkey, kc->salt);
    kc_data_t** rv_ptr = &kc->hash[hashval & hashmask];
    kc_data_t* rv;

    while((rv = *rv_ptr)) {
        if(!memcmp(pubkey, rv->pubkey, 32)) {
            // log_debug("Cache hit!");
            satom_inc(kc->cache_hit);
            rv->clock_data++;
            return rv;
        }
        rv_ptr = &rv->next;
    }

    // log_debug("Cache miss!");
    satom_inc(kc->cache_miss);

    // set rv to an eviction target (or unused array slot
    //  early on, same thing either way).
    // there are more efficient ways to write this...
    const unsigned clock_mask = kc->size - 1;
    uint8_t* da = kc->data_array;
    do {
        rv = (kc_data_t*)&da[kc->data_size * kc->hand++];
        kc->hand &= clock_mask;
        if(!rv->clock_data)
            break;
        rv->clock_data >>= 1U;
    } while(1);

    // If we're re-using an old cache entry,
    // remove victim from the hash index before
    //  we re-use his space and re-insert him via rv_ptr
    if(likely(rv->linked_at)) {
        if(rv->next)
            rv->next->linked_at = rv->linked_at;
        *rv->linked_at = rv->next;
    }

    // store into previously-found hash location
    *rv_ptr = rv;

    // Set up the rv as a new entry for this pubkey
    rv->next = NULL;
    rv->linked_at = rv_ptr;
    rv->shared_init = 0;
    memcpy(rv->pubkey, pubkey, 32);
    return rv;
}

void dnscurve_global_setup(void) {
    if(!gconfig.num_curve_keys)
        return;

    // Suck up all the random bytes the I/O threads will
    //  need later after chroot().  Each thread needs
    //  48 for the nonce generator and 4 for the lookup3 salt.
    unsigned howmany = gconfig.num_io_threads * (48 + 4);
    rpool = malloc(howmany);

    // If /dev/urandom doesn't exist, the current NaCl
    //  randombytes() will sleep indefinitely here.  I suspect
    //  this is one of many reasons why it's not in the main
    //  library yet (another being that /dev/urandom isn't quite
    //  universal).
    randombytes(rpool, howmany);
}

void dnscurve_iothread_setup(dnspacket_context_t* c, const dns_addr_t* addrconf) {
    assert(c); assert(addrconf);

    if(!addrconf->num_curve_keys)
       return;

    c->curve_txt_buf = mmap(NULL, gconfig.max_response, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS, -1, 0);
    load_thread_keys(c, addrconf);
    uint8_t* rbytes = rpool + (c->threadnum * 52);
    init_nonce64(c, rbytes); // uses 48 of our 52 bytes
    rbytes += 48;
    kc_init(c, addrconf, *((uint32_t*)rbytes));
}

static const uint8_t b32_chars[256] = {
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32,

     0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 32, 32, 32, 32, 32, 32,

    32, 32, 10, 11, 12, 32, 13, 14, 15, 32, 16, 17, 18, 19, 20, 32,
    21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 32, 32, 32, 32,

    32, 32, 10, 11, 12, 32, 13, 14, 15, 32, 16, 17, 18, 19, 20, 32,
    21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 32, 32, 32, 32,

    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
};

#define IS_X1A(_p, _o) (\
       ((_p[_o] | 0x20) == 'x')\
    && (_p[_o + 1] == '1')\
    && ((_p[_o + 2] | 0x20) == 'a')\
)

static unsigned decode_curve_qname(unsigned packet_len, const uint8_t* packet, uint8_t** answer, uint8_t* nonce_and_box, uint8_t* pubkey) {
    assert(packet_len >= 146);
    unsigned poffs = 12; // where query name starts
    unsigned decoded = 0;
    unsigned nb_decoded = 0;
    unsigned seen_short_lbl = 0;
    unsigned llen;
    unsigned vbits = 0;
    unsigned v = 0;
    bool pk_label = false;

    uint8_t* output = nonce_and_box;
    while(packet_len && (llen = packet[poffs++])) {
        if(llen > --packet_len)
            return 0;
        packet_len -= llen;

        if(llen == 50) {
            if(seen_short_lbl)
                return 0; // can't have <50 followed by 50
        }
        else if(llen == 54) {
            // 45 is the minimum legal decoded nonce_and_box size:
            //   (12 nonce, 16 box prefix, 17 minimum query size).
            // Also, the vbits, v checks ensure the previous label
            //   ended on an even decoded byte boundary.
            if(decoded > 44 && vbits < 5 && !v && IS_X1A(packet, poffs)) {
                llen = 51;
                poffs += 3;
                vbits = 0;
                pk_label = true;
                nb_decoded = decoded;
                decoded = 0;
                output = pubkey;
            }
            else {
                return 0;
            }
        }
        else if(llen < 50) {
            if(seen_short_lbl++)
                return 0; // can't have 2x <50 either
        }
        else {
            return 0; // 51-53, 55+ are all illegal
        }

        while(llen--) {
            uint8_t val = b32_chars[packet[poffs++]];
            if(val == 32)
                return 0; // non-base32 character in label
            v |= (((unsigned)val) << vbits);
            vbits += 5;
            if(vbits >= 8) {
                output[decoded++] = v;
                vbits -= 8;
                v >>= 8;
            }
        }
 
        if(pk_label) {
            assert(vbits == 7);
            assert(decoded == 31);
            output[decoded] = v;
            // We've just decoded the public key, so now
            //  discard the remaining labels and check for IN TXT
            do {
                if(packet_len < 5)
                    return 0;
                llen = packet[poffs++];
                packet_len--;

                if((llen + 4) > packet_len)
                    return 0;
                packet_len -= llen;
                poffs += llen;
            } while(llen);
            if(memcmp(packet + poffs, "\x00\x10\x00\x01", 4))
                return 0;

            *answer = (uint8_t*)packet + poffs + 4;
            return nb_decoded;
        }
    }

    return 0;
}

static unsigned encode_txt_response(uint8_t* out, uint8_t* nonce_box, unsigned nblen) {
    uint8_t* rdlen_p = out;
    out += 2;
    while(nblen) {
        unsigned to_encode = (nblen > 255) ? 255 : nblen;
        *out++ = to_encode;
        memcpy(out, nonce_box, to_encode);
        out += to_encode;
        nonce_box += to_encode;
        nblen -= to_encode;
    }

    unsigned retval = out - rdlen_p;
    *((uint16_t*)rdlen_p) = htons(retval - 2);
    return retval;
}

// Tries to open a box with all of our keys, using the keycache.
//  If successful, returns a pointer to the shared key to use for
//  encrypting the response box and sets *keyinfo for the outer
//  code to use for per-key nonce sep info, otherwise returns NULL
F_NONNULL
static uint8_t* box_open_multi_cached(dnspacket_context_t* c, uint8_t* out, uint8_t* in, unsigned long long len, uint8_t* nonce, uint8_t* pubkey, io_curvekey_t** keyinfo) {
    assert(c); assert(out); assert(in); assert(len); assert(nonce); assert(pubkey); assert(keyinfo);

    kc_data_t* kc_entry = kc_fetch(c->keycache, pubkey);
    unsigned which_key;
    for(which_key = 0; which_key < c->num_curve_keys; which_key++) {
        uint8_t* sk = &kc_entry->shared[which_key * 32];
        if(kc_entry->shared_init <= which_key) {
            kc_entry->shared_init++;
            crypto_box_curve25519xsalsa20poly1305_beforenm(sk, pubkey, c->curve_keys[which_key].key);
        }
        if(!crypto_box_curve25519xsalsa20poly1305_open_afternm(out, in, len, nonce, sk)) {
            *keyinfo = &c->curve_keys[which_key];
            return sk;
        }
    }
    return NULL;
}

F_NONNULL
static unsigned int dnscurve_streamed(dnspacket_context_t* c, const anysin_t* asin, uint8_t* packet, const unsigned int packet_len) {
    assert(c); assert(asin); assert(packet);

    assert(packet_len >= 69);

    satom_inc(&c->stats->curve_strm);

    uint8_t pubkey[32];
    uint8_t nonce[24];
    memcpy(pubkey, &packet[8], 32);
    memcpy(nonce, &packet[40], 12);
    memset(&nonce[12], 0, 12);

    // The box starts at +52, but we need to zero out
    //   a 16 byte prefix (at +36) and pass that to box_open.
    memset(packet + 36, 0, 16);

    // Shifting backwards from +36 to +16 during decrypt sets everything
    //  up just right for the wire response packet to start at +0 again.
    io_curvekey_t* keyinfo;
    uint8_t* sharedkey = box_open_multi_cached(c, packet + 16, packet + 36, packet_len - 36, nonce, pubkey, &keyinfo);
    if(!sharedkey) {
        satom_inc(&c->stats->curve_strm_fail);
        return 0;
    }

    unsigned outlen = process_dns_query(c, asin, packet + 16 + 32, packet_len - (36 + 32));
    if(!outlen)
        return 0;

    noncex96(c->n64_state, keyinfo->sep_mask, keyinfo->sep_val, nonce + 12);
    crypto_box_curve25519xsalsa20poly1305_afternm(packet + 16, packet + 16, outlen + 32, nonce, sharedkey);
    memcpy(packet, "R6fnvWJ8", 8);
    memcpy(packet + 8, nonce, 24);
    return outlen + 48;
}

unsigned int dnscurve_filter(dnspacket_context_t* c, const anysin_t* asin, uint8_t* packet, const unsigned int packet_len) {

    // Min size for a streamlined DNSCurve request is 69 (and TXT is even bigger)
    if(!c->num_curve_keys || packet_len < 69)
        return process_dns_query(c, asin, packet, packet_len);

    // Streamlined.  "Q6fnvWj8" can only be DNSCurve.  Interpreting
    //   that as the first 8 bytes of a regular DNS query would mean
    //   that the query's QR bit was set, among other undesirable things.
    if(!memcmp(packet, "Q6fnvWj8", 8))
        return dnscurve_streamed(c, asin, packet, packet_len);

    // From here out is TXT-format DNSCurve handling.
    // It's wrapped in a do/while(0), and any failure of TXT-format
    // DNSCurve decoding results in a "break" which falls out
    // to attempting to handle the packet as a plain old DNS
    // query at the end.

/*
  Minimum size of any legal DNS Query:
  17 bytes:
    12 bytes: Normal DNS Header
    1 byte: Root Query Name (\0)
    4 bytes: type, class

  Minimum size of a TXT-format DNSCurve Query:
  146 bytes:
    12 bytes: Normal DNS header
    130 bytes: Query Name (74 + 55 + 1)
      74 bytes: labelize(72 bytes) (as two labels: 50, 22)
        72 bytes: base32(12 byte nonce + 33 byte box)
          33 byte box: 17 byte min query + 16 bytes box prefix
      55 bytes: 54-byte label: base32 client public key w/ x1a prefix
      1 byte: \0
    4 bytes: TXT, IN

  Since most plain DNS query packets are under the 146 mark, this
   serves as a convenient fast-path for differentiation, so it's the first
   thing we check for.
*/

    do {
        if(packet_len < 146) break; // Too short
        if(packet[2] & ~1U) break;  // Bad flags1 bits
        if(memcmp(&packet[3], "\000\000\001", 3)) break; // Bad flags 2 or qdcount != 1

        uint8_t pubkey[32];
        uint8_t nonce[24];
        uint8_t* packet_answer;
        unsigned nonce_box_len = decode_curve_qname(packet_len, packet, &packet_answer,
            c->curve_txt_buf + 4, pubkey);
        if(!nonce_box_len) break; // decode failure
        assert(packet_answer);

        // If we got this far and decoder didn't see anything wrong with the qname,
        //  we're going to go ahead and count it (and count it as a failure if box
        //  opening fails).  But we'll still try to fallback to normal processing
        //  on failure.
        satom_inc(&c->stats->curve_txt);
        memcpy(nonce, c->curve_txt_buf + 4, 12);
        memset(&nonce[12], 0, 12);
        memset(c->curve_txt_buf, 0, 16);

        io_curvekey_t* keyinfo;
        uint8_t* sharedkey = box_open_multi_cached(c, c->curve_txt_buf, c->curve_txt_buf, nonce_box_len + 4, nonce, pubkey, &keyinfo);
        if(!sharedkey) {
            satom_inc(&c->stats->curve_txt_fail);
            return 0;
        }

        unsigned outlen = process_dns_query(c, asin, c->curve_txt_buf + 32, nonce_box_len - 28);
        if(!outlen)
            return 0;

        noncex96(c->n64_state, keyinfo->sep_mask, keyinfo->sep_val, nonce + 12);
        crypto_box_curve25519xsalsa20poly1305_afternm(c->curve_txt_buf, c->curve_txt_buf, outlen + 32, nonce, sharedkey);

        // QR-bit, 1 answer, 0 auth, 0 addtl, leave rest
        //  as the client had it (query-id, RD-bit, etc - we
        //  validated most of the flags to be zero'd earlier anyways)
        packet[2] |= 0x80;
        memcpy(packet + 6, "\x00\x01\x00\x00\x00\x00", 6);

        // TXT RR Header (compress against query name, class IN, type TXT, TTL 0)
        memcpy(packet_answer, "\xC0\x0C\x00\x10\x00\x01\x00\x00\x00\x00", 10);
        packet_answer += 10;

        // copy nonce ext into the empty 16 bytes before the box in curve_txt_buf
        memcpy(c->curve_txt_buf + 4, &nonce[12], 12);

        // encode nonce ext + box as TXT rdata straight into the original packet buffer
        packet_answer += encode_txt_response(packet_answer, c->curve_txt_buf + 4, 12 + outlen + 16);
        return packet_answer - packet;
    } while(0);

    //  Plain DNS
    return process_dns_query(c, asin, packet, packet_len);
}
