/* Copyright © 2011 Brandon L Black <blblack@gmail.com>
 *
 * This file is part of gdnsd.
 *
 * gdnsd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * gdnsd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with gdnsd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "config.h"

#include <stdbool.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <stdarg.h>
#include <time.h>
#include <pthread.h>
#include <errno.h>
#include <assert.h>

#include "dmn.h"

#if ! HAVE_DECL_FPUTS_UNLOCKED
#define fputs_unlocked fputs
#endif

// Gets set to true near the end of dmn_daemonize()
extern bool dmn_daemonized;
bool dmn_daemonized = false;

// When the daemon is built in debug mode (!defined NDEBUG),
//  *and* this flag is set to true by the daemon,
//  dmn_log_debug() emits output
bool dmn_debug = false;

// Length of the whole format buffer
#ifndef DMN_FMTBUF_SIZE
#define DMN_FMTBUF_SIZE 4096U
#endif

// Log message prefixes when using stderr
static const char* pfx_debug = "debug: ";
static const char* pfx_info = "info: ";
static const char* pfx_warning = "warning: ";
static const char* pfx_err = "error: ";
static const char* pfx_crit = "fatal: ";
static const char* pfx_unknown = "???: ";

/*********************************************************************/
/*** fmtbuf code *****************************************************/
/*********************************************************************/

typedef struct {
    unsigned used;
    char buf[DMN_FMTBUF_SIZE];
} fmtbuf_t;

static pthread_key_t fmtbuf_key;
static pthread_once_t fmtbuf_key_once = PTHREAD_ONCE_INIT;

static void fmtbuf_make_key(void) { pthread_key_create(&fmtbuf_key, NULL); }

static fmtbuf_t* fmtbuf_get(void) {
    pthread_once(&fmtbuf_key_once, fmtbuf_make_key);
    fmtbuf_t* fmtbuf;
    if((fmtbuf = pthread_getspecific(fmtbuf_key)) == NULL) {
        fmtbuf = calloc(1, sizeof(fmtbuf_t));
        pthread_setspecific(fmtbuf_key, (void*)fmtbuf);
    }

    return fmtbuf;
}

char* dmn_fmtbuf_alloc(unsigned size) {
    fmtbuf_t* fmtbuf = fmtbuf_get();
    if(fmtbuf->used + size > DMN_FMTBUF_SIZE)
        dmn_log_fatal("BUG: format buffer exhausted");
    char* retval = &fmtbuf->buf[fmtbuf->used];
    fmtbuf->used += size;
    return retval;
}

/**********************************************************************
* dmn_strerror(), which hides GNU or POSIX strerror_r() thread-safe
* errno->string translation behind a more strerror()-like interface
* using dmn_fmtbuf_alloc()
***********************************************************************/

#define ERRNO_MAXLEN 256 // Max length of an errno string
const char* dmn_strerror(const int errnum) {
    char* buf = dmn_fmtbuf_alloc(ERRNO_MAXLEN);

#ifdef STRERROR_R_CHAR_P
    // GNU-style
    buf = strerror_r(errnum, buf, ERRNO_MAXLEN);
#else
    // POSIX style
    if(strerror_r(errnum, buf, ERRNO_MAXLEN)) {
        if(errno == EINVAL)
            snprintf(buf, 256, "Invalid errno: %i", errnum);
        else
            dmn_log_fatal("strerror_r(,,256) failed");
    }
#endif

    return buf;
}

/*****************************************************************/
/*** The core logging funcs: dmn_loggerv and dmn_logger **********/
/*****************************************************************/

void dmn_loggerv(int level, const char* fmt, va_list ap) {
    assert(fmt);

    if(dmn_daemonized) {
        vsyslog(level, fmt, ap);
    }
    else {
        time_t t = time(NULL);
        struct tm tmp;
        localtime_r(&t, &tmp);
        char tstamp[10];
        if(!strftime(tstamp, 10, "%T ", &tmp))
            strcpy(tstamp, "--:--:-- ");

        va_list apcopy;
        va_copy(apcopy, ap);
        const char* pfx;
        switch(level) {
            case LOG_DEBUG: pfx = pfx_debug; break;
            case LOG_INFO: pfx = pfx_info; break;
            case LOG_WARNING: pfx = pfx_warning; break;
            case LOG_ERR: pfx = pfx_err; break;
            case LOG_CRIT: pfx = pfx_crit; break;
            default: pfx = pfx_unknown; break;
        }
        flockfile(stderr);
        fputs_unlocked(tstamp, stderr);
        fputs_unlocked(pfx, stderr);
        vfprintf(stderr, fmt, apcopy);
        putc_unlocked('\n', stderr);
        funlockfile(stderr);
        va_end(apcopy);
    }

    // Reset (free allocations within) the format buffer
    fmtbuf_get()->used = 0;
}

void dmn_logger(int level, const char* fmt, ...) {
    assert(fmt);
    va_list ap;
    va_start(ap, fmt);
    dmn_loggerv(level, fmt, ap);
    va_end(ap);
}
