/* Copyright © 2010 Brandon L Black <blblack@gmail.com>
 *
 * This file is part of gdnsd.
 *
 * gdnsd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * gdnsd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with gdnsd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "monio.h"
#include "dnsio_tcp.h"

#include <string.h>
#include <unistd.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <fcntl.h>

typedef struct {
    const char* name;
    const char* url_path; // Def: "/"
    const char* vhost;    // Def: NULL (no "Host:" header)
    const char* req_data;  // Constructed from url_path / vhost
    unsigned req_data_len; // length of above
    unsigned port;        // Def: 80 Range: 1-65535
    unsigned up_thresh;   // Def: 20 Range: 1-uintmax
    unsigned ok_thresh;   // Def: 10 Range: 1-uintmax
    unsigned down_thresh; // Def: 10 Range: 1-uintmax
    unsigned interval;    // Def: 10 Range: 1-3600
    unsigned timeout;     // Def: 3  Range: 1-300, also, must be < interval
} service_type_t;

typedef enum {
    MONIO_STATE_WAITING = 0,   // waiting for interval to expire before next send
    MONIO_STATE_WRITING,   // trying to send the request
    MONIO_STATE_READING  // trying to receive the response
} monio_event_state_t;

typedef struct {
    service_type_t* svc_type;
    const char* desc;
    monio_state_t** monio_state_ptrs;
    ev_io* read_watcher;
    ev_io* write_watcher;
    ev_timer* timeout_watcher;
    ev_timer* interval_watcher;
    anysin_t asin; // anysin_t and IPv6 support
    unsigned num_state_ptrs;
    char res_buf[14];
    int sock;
    unsigned n_failure;
    unsigned n_success;
    monio_event_state_t state;
    unsigned done;
    bool already_connected;
} monio_data_t;

#define monio_state_set(x,y) satom_set(x,y)
#define MONIO_STATE_UNINIT 0

static int max_stats_len = 0;
static unsigned int num_mons = 0;
static monio_data_t** mons = NULL;

/*
=== State Monitoring ===

state: enum(UNINIT, UP, DANGER, DOWN), init to UNINIT
n_failure: uint, init to 0
n_success: uint, init to 0

state_updater {
    response_is_OK:
        state == UP      => NO-OP;
        state == DANGER  => if(++n_success == ok_thresh) state = UP;
        state == DOWN    => if(++n_success == up_thresh) state = UP;
        state == UNINIT  => state = UP;
    response_is_BAD:
        n_success = 0;
        state == UP      => n_failure = 1; state = DANGER;
        state == DANGER  => if(++n_failure == down_thresh) state = DOWN;
        state == DOWN    => NO-OP;
        state == UNINIT  => state = DOWN;
}
*/

F_NONNULL
static void monio_state_updater(monio_data_t* md, const bool latest_status) {
    assert(md);

    md->state = MONIO_STATE_WAITING;
    monio_state_uint_t now_state = monio_state_get(md->monio_state_ptrs[0]);

    if(latest_status) { // New Success
        switch(now_state) {
            case MONIO_STATE_UP:
                break;
            case MONIO_STATE_DANGER:
                if(++(md->n_success) == md->svc_type->ok_thresh) {
                    log_info("'%s' transitioned to the UP state", md->desc);
                    for(unsigned i = 0; i < md->num_state_ptrs; i++)
                        monio_state_set(md->monio_state_ptrs[i], MONIO_STATE_UP);
                }
                break;
            case MONIO_STATE_DOWN:
                if(++(md->n_success) == md->svc_type->up_thresh) {
                    log_info("'%s' transitioned to the UP state", md->desc);
                    for(unsigned i = 0; i < md->num_state_ptrs; i++)
                        monio_state_set(md->monio_state_ptrs[i], MONIO_STATE_UP);
                }
                break;
            case MONIO_STATE_UNINIT:
                log_info("'%s' initialized to the UP state", md->desc);
                for(unsigned i = 0; i < md->num_state_ptrs; i++)
                    monio_state_set(md->monio_state_ptrs[i], MONIO_STATE_UP);
                break;
        }
    }
    else { // New Failure
        md->n_success = 0;
        switch(now_state) {
            case MONIO_STATE_UP:
                md->n_failure = 1;
                log_info("'%s' transitioned to the DANGER state", md->desc);
                for(unsigned i = 0; i < md->num_state_ptrs; i++)
                    monio_state_set(md->monio_state_ptrs[i], MONIO_STATE_DANGER);
                break;
            case MONIO_STATE_DANGER:
                if(++(md->n_failure) == md->svc_type->down_thresh) {
                     log_info("'%s' transitioned to the DOWN state", md->desc);
                    for(unsigned i = 0; i < md->num_state_ptrs; i++)
                        monio_state_set(md->monio_state_ptrs[i], MONIO_STATE_DOWN);
                }
                break;
            case MONIO_STATE_DOWN:
                break;
            case MONIO_STATE_UNINIT:
                log_info("'%s' initialized to the DOWN state", md->desc);
                for(unsigned i = 0; i < md->num_state_ptrs; i++)
                    monio_state_set(md->monio_state_ptrs[i], MONIO_STATE_DOWN);
                break;
        }
    }
}

F_NONNULL
static void monio_interval_cb(struct ev_loop* loop, struct ev_timer* t, const int revents V_UNUSED) {
    assert(loop); assert(t);
    assert(revents == EV_TIMER);

    monio_data_t* md = (monio_data_t*)t->data;

    assert(md);
    assert(md->sock == -1);

    if(unlikely(md->state != MONIO_STATE_WAITING)) {
        log_warn("A monitoring request attempt seems to have "
            "lasted longer than the monitoring interval. "
            "Skipping this round of monitoring - are you "
            "starved for CPU time?");
        return;
    }

    assert(!ev_is_active(md->read_watcher));
    assert(!ev_is_active(md->write_watcher));
    assert(!ev_is_active(md->timeout_watcher));

    log_debug("Starting state poll of '%s'", md->desc);

    do {
        const int sock = socket(PF_INET, SOCK_STREAM, gdnsd_getproto_tcp());
        if(unlikely(sock == -1)) {
            log_err("Failed to create monitoring socket: %s", logf_errno());
            break;
        }

        if(unlikely(fcntl(sock, F_SETFL, (fcntl(sock, F_GETFL, 0)) | O_NONBLOCK) == -1)) {
            log_err("Failed to set O_NONBLOCK on monitoring socket: %s", logf_errno());
            close(sock);
            break;
        }

        md->already_connected = true;
        unsigned asize = md->asin.sa.sa_family == AF_INET
            ? sizeof(struct sockaddr_in)
            : sizeof(struct sockaddr_in6);
        if(likely(connect(sock, (void*)&md->asin, asize) == -1)) {
            if(likely(errno == EINPROGRESS)) { md->already_connected = false; }
            else {
                switch(errno) {
                    case EPIPE:
                    case ECONNREFUSED:
                    case ETIMEDOUT:
                    case EHOSTUNREACH:
                    case EHOSTDOWN:
                    case ENETUNREACH:
                        break;
                    default:
                        log_err("Failed to connect() monitoring socket to remote server, possible local problem: %s", logf_errno());
                }
                close(sock);
                break;
            }
        }

        md->sock = sock;
        md->state = MONIO_STATE_WRITING;
        md->done = 0;
        ev_io_set(md->write_watcher, sock, EV_WRITE);
        ev_io_start(loop, md->write_watcher);
        ev_timer_set(md->timeout_watcher, md->svc_type->timeout, 0);
        ev_timer_start(loop, md->timeout_watcher);
        return;
    } while(0);

    // This is only reachable via "break"'s above, which indicate an immediate failure
    log_debug("State poll of '%s' failed very quickly", md->desc);
    monio_state_updater(md, false);
}

F_NONNULL
static void monio_write_cb(struct ev_loop* loop, struct ev_io* io, const int revents V_UNUSED) {
    assert(loop); assert(io);
    assert(revents == EV_WRITE);

    monio_data_t* md = (monio_data_t*)io->data;

    assert(md);
    assert(md->state == MONIO_STATE_WRITING);
    assert(!ev_is_active(md->read_watcher));
    assert(ev_is_active(md->write_watcher));
    assert(md->sock != -1);

    int sock = md->sock;
    if(likely(!md->already_connected)) {
        // nonblocking connect() just finished, need to check status
        int so_error = 0;
        unsigned int so_error_len = sizeof(so_error);
        getsockopt(sock, SOL_SOCKET, SO_ERROR, &so_error, &so_error_len);
        if(unlikely(so_error)) {
            switch(so_error) {
                case EPIPE:
                case ECONNREFUSED:
                case ETIMEDOUT:
                case EHOSTUNREACH:
                case EHOSTDOWN:
                case ENETUNREACH:
                    break;
                default:
                    log_err("Failed to connect() monitoring socket to remote server, possible local problem: %s", logf_errnum(so_error));
            }

            log_debug("State poll of '%s' failed quickly: %s", md->desc, logf_errnum(so_error));
            close(sock); md->sock = -1;
            ev_io_stop(loop, md->write_watcher);
            ev_timer_stop(loop, md->timeout_watcher);
            monio_state_updater(md, false);
            return;
        }
        md->already_connected = true;
    }

    const unsigned to_send = md->svc_type->req_data_len - md->done;
    const int sent = send(sock, md->svc_type->req_data + md->done, md->svc_type->req_data_len, 0);
    if(unlikely(sent == -1)) {
        switch(errno) {
            case EAGAIN:
            case EINTR:
                return;
            case ENOTCONN:
            case ECONNRESET:
            case ETIMEDOUT:
            case EHOSTUNREACH:
            case ENETUNREACH:
            case EPIPE:
                break;
            default:
                log_err("write() to monitoring socket failed, possible local problem: %s", logf_errno());
        }
        shutdown(sock, SHUT_RDWR);
        close(sock);
        md->sock = -1;
        ev_io_stop(loop, md->write_watcher);
        ev_timer_stop(loop, md->timeout_watcher);
        monio_state_updater(md, false);
    }
    if(unlikely(sent != (signed)to_send)) {
        md->done += sent;
        return;
    }

    md->done = 0;
    md->state = MONIO_STATE_READING;
    ev_io_stop(loop, md->write_watcher);
    ev_io_set(md->read_watcher, sock, EV_READ);
    ev_io_start(loop, md->read_watcher);
}

F_NONNULL
static void monio_read_cb(struct ev_loop* loop, struct ev_io* io, const int revents V_UNUSED) {
    assert(loop); assert(io);
    assert(revents == EV_READ);

    monio_data_t* md = (monio_data_t*)io->data;

    assert(md);
    assert(md->state == MONIO_STATE_READING);
    assert(ev_is_active(md->read_watcher));
    assert(!ev_is_active(md->write_watcher));
    assert(md->sock != -1);

    bool final_status = false;
    const int to_recv = 13 - md->done;
    const int recvd = recv(md->sock, md->res_buf + md->done, to_recv, 0);
    if(unlikely(recvd == -1)) {
        switch(errno) {
            case EAGAIN:
            case EINTR:
                return;
            case ETIMEDOUT:
            case ENOTCONN:
            case ECONNRESET:
            case EPIPE:
                break;
            default:
                log_err("read() from monitoring socket failed, possible local problem: %s", logf_errno());
        }
    }
    else if(recvd < to_recv) {
        md->done += recvd;
        return;
    }
    else {
        // Some webservers respond as HTTP/1.1 even for HTTP/1.0 requests
        final_status = !strncmp(md->res_buf, "HTTP/1.0 200 ", 13)
            || !strncmp(md->res_buf, "HTTP/1.1 200 ", 13);
    }

    // I don't believe we actually need to read the rest of the response before
    //   shutdown/close in order to avoid bad TCP behavior, but I could be wrong.

    log_debug("State poll of '%s' %s", md->desc, final_status ? "succeeded" : "failed");
    shutdown(md->sock, SHUT_RDWR);
    close(md->sock);
    md->sock = -1;
    ev_io_stop(loop, md->read_watcher);
    ev_timer_stop(loop, md->timeout_watcher);
    monio_state_updater(md, final_status);
}

F_NONNULL
static void monio_timeout_cb(struct ev_loop* loop, struct ev_timer* t, const int revents V_UNUSED) {
    assert(loop); assert(t);
    assert(revents == EV_TIMER);

    monio_data_t* md = (monio_data_t*)t->data;

    assert(md);
    assert(md->sock != -1);
    assert(
        (md->state == MONIO_STATE_READING && ev_is_active(md->read_watcher))
     || (md->state == MONIO_STATE_WRITING && ev_is_active(md->write_watcher))
    );

    log_debug("State poll of '%s' timed out", md->desc);
    if(md->state == MONIO_STATE_READING) ev_io_stop(loop, md->read_watcher);
    else if(md->state == MONIO_STATE_WRITING) ev_io_stop(loop, md->write_watcher);
    if(md->sock != -1) {
        shutdown(md->sock, SHUT_RDWR);
        close(md->sock); md->sock = -1;
    }
    monio_state_updater(md, false);
}

// Called once after all resources are monio_add()'d,
//  from main thread.  mon_loop happens to be the default
//  loop currently, and should be empty of events at
//  this point so that we can fall out after the initial
//  round of monitoring.
void monio_start(struct ev_loop* mon_loop) {
    assert(mon_loop);

    // Fall out quickly if nothing to monitor
    if(!num_mons) return;

    for(unsigned int i = 0; i < num_mons; i++) {
        ev_timer* ival_watcher = mons[i]->interval_watcher;
        ev_timer_set(ival_watcher, 0, 0);
        ev_timer_start(mon_loop, ival_watcher);
    }

    // Run the loop once until all events drain, which will
    // be one full monitoring cycle of each resource (without
    // any artificial delays).
    log_info("Starting initial round of monitoring ...");
    ev_run(mon_loop, 0);
    log_info("Initial round of monitoring complete");

    // Now set up the permanent recurring monitoring intervals
    //  before returning control to main
    for(unsigned int i = 0; i < num_mons; i++) {
        monio_data_t* mon = mons[i];
        const unsigned ival = mon->svc_type->interval;
        const double stagger = (((double)i) / ((double)num_mons)) * ((double)ival);
        ev_timer* ival_watcher = mon->interval_watcher;
        ev_timer_set(ival_watcher, stagger, ival);
        ev_timer_start(mon_loop, ival_watcher);
    }
}

// We only have to check the address, because the port
//  is determined by service_type.
static bool addr_eq(const anysin_t* a, const anysin_t* b) {
    if(a->sa.sa_family == b->sa.sa_family) {
        if(a->sa.sa_family == AF_INET) {
            if(a->sin.sin_addr.s_addr == b->sin.sin_addr.s_addr)
                 return true;
        }
        else {
            assert(a->sa.sa_family == AF_INET6);
            if(a->sin6.sin6_addr.s6_addr == b->sin6.sin6_addr.s6_addr)
                return true;
        }
    }
    return false;
}

static unsigned num_svc_types = 0;
static service_type_t* service_types;
static const vscf_data_t* service_types_vscf;

// Called by plugins once per monitored service_type+IP combination
//  during their _load_config() phase (very important)
//  to request monitoring and initialize various data/state.
void monio_add(const char* svctype_name, const char* desc, const char* addr, monio_state_t* monio_state_ptr) {
    monio_state_set(monio_state_ptr, MONIO_STATE_UNINIT);

    monio_data_t* this_monio = calloc(1, sizeof(monio_data_t));

    {
        int status;
        if(!strchr(addr, ':')) { // No colons == IPv4
            this_monio->asin.sa.sa_family = AF_INET;
            status = inet_pton(AF_INET, addr, &this_monio->asin.sin.sin_addr);
        }
        else { // Colons == assumed IPv6
#if USE_V6SOCK
            this_monio->asin.sa.sa_family = AF_INET6;
            status = inet_pton(AF_INET6, addr, &this_monio->asin.sin6.sin6_addr);
#else
            log_fatal("This build does not support IPv6 for monitor address spec '%s'", addr);
#endif
        }
        if(status < 1)
            log_fatal("Address '%s' invalid in monitoring request for '%s'", addr, desc);
    }

    if(!svctype_name || !strcmp(svctype_name, "default")) {
        this_monio->svc_type = &service_types[num_svc_types];
    }
    else {
        signed idx = vscf_hash_get_index_bykey(service_types_vscf, svctype_name, strlen(svctype_name));
        if(idx < 0)
            log_fatal("Invalid service_type '%s' in monitoring request for '%s'", svctype_name, desc);
        this_monio->svc_type = &service_types[idx];
    }

    if(this_monio->asin.sa.sa_family == AF_INET)
        this_monio->asin.sin.sin_port = htons(this_monio->svc_type->port);
    else
        this_monio->asin.sin6.sin6_port = htons(this_monio->svc_type->port);

    // now check for uniqueness
    for(unsigned i = 0; i < num_mons; i++) {
        monio_data_t* that_monio = mons[i];
        if(addr_eq(&this_monio->asin, &that_monio->asin) && this_monio->svc_type == that_monio->svc_type) {
            // We found a duplicate
            free(this_monio);
            that_monio->monio_state_ptrs = realloc(
                that_monio->monio_state_ptrs,
                (that_monio->num_state_ptrs + 1) * sizeof(monio_state_t*)
            );
            that_monio->monio_state_ptrs[that_monio->num_state_ptrs++] = monio_state_ptr;
            return;
        }
    }

    this_monio->desc = strdup(desc);
    this_monio->num_state_ptrs = 1;
    this_monio->monio_state_ptrs = malloc(sizeof(monio_state_t*));
    this_monio->monio_state_ptrs[0] = monio_state_ptr;
    this_monio->n_failure = 0;
    this_monio->n_success = 0;
    this_monio->state = MONIO_STATE_WAITING;
    this_monio->sock = -1;

    this_monio->read_watcher = malloc(sizeof(ev_io));
    ev_io_init(this_monio->read_watcher, &monio_read_cb, -1, 0);
    this_monio->read_watcher->data = this_monio;

    this_monio->write_watcher = malloc(sizeof(ev_io));
    ev_io_init(this_monio->write_watcher, &monio_write_cb, -1, 0);
    this_monio->write_watcher->data = this_monio;

    this_monio->timeout_watcher = malloc(sizeof(ev_timer));
    ev_timer_init(this_monio->timeout_watcher, &monio_timeout_cb, 0, 0);
    this_monio->timeout_watcher->data = this_monio;

    this_monio->interval_watcher = malloc(sizeof(ev_timer));
    ev_timer_init(this_monio->interval_watcher, &monio_interval_cb, 0, 0);
    this_monio->interval_watcher->data = this_monio;

    mons = realloc(mons, sizeof(monio_data_t*) * (num_mons + 1));
    mons[num_mons++] = this_monio;
}

F_NONNULL
static void make_req_data(service_type_t* s) {
    assert(s); assert(s->url_path);
    unsigned url_len = strlen(s->url_path);
    if(s->vhost) {
        s->req_data_len = 25 + url_len + strlen(s->vhost);
        char* req_data = malloc(s->req_data_len + 1);
        snprintf(req_data, s->req_data_len + 1, "GET %s HTTP/1.0\r\nHost: %s\r\n\r\n", s->url_path, s->vhost);
        s->req_data = req_data;
    }
    else {
        s->req_data_len = 17 + url_len;
        char* req_data = malloc(s->req_data_len + 1);
        snprintf(req_data, s->req_data_len + 1, "GET %s HTTP/1.0\r\n\r\n", s->url_path);
        s->req_data = req_data;
    }
}

#define SVC_OPT_UINT(_hash, _typnam, _loc, _min, _max) \
    do { \
        const vscf_data_t* _data = vscf_hash_get_data_byconstkey(_hash, #_loc, true); \
        if(_data) { \
            unsigned long _val; \
            if(!vscf_is_simple(_data) \
            || !vscf_simple_get_as_ulong(_data, &_val)) \
                log_fatal("Service type '%s': option '%s': Value must be a positive integer", _typnam, #_loc); \
            if(_val < _min || _val > _max) \
                log_fatal("Service type '%s': option '%s': Value out of range (%lu, %lu)", _typnam, #_loc, _min, _max); \
            this_svc->_loc = (unsigned) _val; \
        } \
    } while(0)

#define SVC_OPT_STR(_hash, _typnam, _loc) \
    do { \
        const vscf_data_t* _data = vscf_hash_get_data_byconstkey(_hash, #_loc, true); \
        if(_data) { \
            if(!vscf_is_simple(_data)) \
                log_fatal("Service type '%s': option %s: Wrong type (should be string)", _typnam, #_loc); \
            this_svc->_loc = strdup(vscf_simple_get_data(_data)); \
        } \
    } while(0)

F_NONNULL
static bool bad_svc_opt(const char* key, unsigned klen V_UNUSED, const vscf_data_t* d V_UNUSED, void* data) {
    assert(key); assert(data);
    log_fatal("Service type '%s', bad option '%s'", (const char*)data, key);
}

void monio_add_servicetypes(const vscf_data_t* svctypes_cfg) {
    if(svctypes_cfg) {
        if(!vscf_is_hash(svctypes_cfg))
            log_fatal("service_types, if defined, must have a hash value");
        num_svc_types = vscf_hash_get_len(svctypes_cfg);
        service_types_vscf = svctypes_cfg;
    }

    service_types = malloc((num_svc_types + 1) * sizeof(service_type_t));
    service_types[num_svc_types].name = "default";
    service_types[num_svc_types].url_path = "/";
    service_types[num_svc_types].vhost = NULL;
    service_types[num_svc_types].port = 80;
    service_types[num_svc_types].up_thresh = 20;
    service_types[num_svc_types].ok_thresh = 10;
    service_types[num_svc_types].down_thresh = 10;
    service_types[num_svc_types].interval = 10;
    service_types[num_svc_types].timeout = 3;

    make_req_data(&service_types[num_svc_types]);

    if(!num_svc_types) return;

    for(unsigned i = 0; i < num_svc_types; i++) {
        service_type_t* this_svc = &service_types[i];
        this_svc->name = strdup(vscf_hash_get_key_byindex(svctypes_cfg, i, NULL));
        if(!strcmp(this_svc->name, "default"))
            log_fatal("Explicit service_type name 'default' not allowed");
        const vscf_data_t* svctype_cfg = vscf_hash_get_data_byindex(svctypes_cfg, i);
        if(!vscf_is_hash(svctype_cfg))
            log_fatal("Definition of service type '%s' must be a hash", this_svc->name);

        this_svc->url_path = "/";
        this_svc->vhost = NULL;
        this_svc->port = 80;
        this_svc->up_thresh = 20;
        this_svc->ok_thresh = 10;
        this_svc->down_thresh = 10;
        this_svc->interval = 10;
        this_svc->timeout = 3;
        SVC_OPT_STR(svctype_cfg, this_svc->name, url_path);
        SVC_OPT_STR(svctype_cfg, this_svc->name, vhost);
        SVC_OPT_UINT(svctype_cfg, this_svc->name, port, 1LU, 65535LU);
        SVC_OPT_UINT(svctype_cfg, this_svc->name, up_thresh, 1LU, 65535LU);
        SVC_OPT_UINT(svctype_cfg, this_svc->name, ok_thresh, 1LU, 65535LU);
        SVC_OPT_UINT(svctype_cfg, this_svc->name, down_thresh, 1LU, 65535LU);
        SVC_OPT_UINT(svctype_cfg, this_svc->name, interval, 1LU, 3600LU);
        SVC_OPT_UINT(svctype_cfg, this_svc->name, timeout, 1LU, 300LU);
        if((double)this_svc->timeout > (double)this_svc->interval * 0.9)
            log_fatal("Service type '%s': timeout must be less than 90%% of interval)", this_svc->name);
        make_req_data(this_svc);
        vscf_hash_iterate(svctype_cfg, true, bad_svc_opt, (void*)this_svc->name);
    }
}

static const char http_head[] = "<span class='bold big'>Monitored Service States:</span><table>\r\n"
    "<tr><th>Service</th><th>State</th></tr>\r\n";
static unsigned http_head_len = sizeof(http_head) - 1;

static const char http_tmpl[] = "<tr><td>%s</td><td class='%s'>%s</td></tr>\r\n";
static unsigned http_tmpl_len = sizeof(http_head) - 7;

static const char http_foot[] = "</table>\r\n";
static unsigned http_foot_len = sizeof(http_foot) - 1;

static const char csv_head[] = "Service,State\r\n";
static unsigned csv_head_len = sizeof(csv_head) - 1;

static const char csv_tmpl[] = "%s,%s\r\n";

static const char* state_txt[4] = {
    "UNINIT", // should be unused in practice due to startup ordering
    "DOWN",
    "DANGER",
    "UP",
};

// statio calls this at the appropriate time (long after all
//  basic setup is done, but before monio_start() time).
// monio's job here is to inform statio of the maximum possible
//  size of its stats output (csv or html, although html is probably
//  always the larger of the two).
unsigned monio_get_max_stats_len(void) {
    if(!num_mons) return max_stats_len = 0;

    unsigned retval = http_head_len + http_foot_len
           + (num_mons * (http_tmpl_len + (6*2))); // 6 is len(DANGER)
    for(unsigned i = 0; i < num_mons; i++)
        retval += strlen(mons[i]->desc);

    return max_stats_len = retval;
}

// Output our stats in html form to buf, returning
//  how many characters we added to the buf.
unsigned monio_stats_out_html(char* buf) {
    assert(buf);

    if(!num_mons) return 0;
    assert(max_stats_len);

    const char* const buf_start = buf;
    int avail = max_stats_len;

    memcpy(buf, http_head, http_head_len);
    buf += http_head_len;
    avail -= http_head_len;

    for(unsigned i = 0; i < num_mons; i++) {
        monio_state_uint_t st = monio_state_get(mons[i]->monio_state_ptrs[0]);
        int written = snprintf(buf, avail, http_tmpl, mons[i]->desc, state_txt[st], state_txt[st]);
        if(unlikely(written >= avail || avail < (int)http_foot_len))
            log_fatal("BUG: monio stats buf miscalculated");
        buf += written;
        avail -= written;
    }

    memcpy(buf, http_foot, http_foot_len);
    buf += http_foot_len;

    return (buf - buf_start);
}

// Output our stats in csv form to buf, returning
//  how many characters we added to the buf.
unsigned monio_stats_out_csv(char* buf) {
    assert(buf);

    if(!num_mons) return 0;
    assert(max_stats_len);

    const char* const buf_start = buf;
    int avail = max_stats_len;

    memcpy(buf, csv_head, csv_head_len);
    buf += csv_head_len;
    avail -= csv_head_len;

    for(unsigned i = 0; i < num_mons; i++) {
        monio_state_uint_t st = monio_state_get(mons[i]->monio_state_ptrs[0]);
        int written = snprintf(buf, avail, csv_tmpl, mons[i]->desc, state_txt[st]);
        if(unlikely(written >= avail))
            log_fatal("BUG: monio stats buf miscalculated");
        buf += written;
        avail -= written;
    }

    return (buf - buf_start);
}
