/* Copyright © 011 Logitech
 *
 * This file is part of gdnsd.
 *
 * gdnsd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * gdnsd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with gdnsd.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Brandon L Black <blblack@gmail.com>
 */

#define GDNSD_PLUGIN_NAME multifo

#include "config.h"
#include <gdnsd-plugin.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

typedef struct {
    uint32_t ipv4;
    monio_state_t state;
} addrstate_t;

typedef struct {
    const char* name;
    unsigned count;
    addrstate_t* addrstates;
} mfo_resource_t;

static mfo_resource_t* resources = NULL;
static unsigned num_resources = 0;

static monio_list_t monio_list = { 0, NULL };

/*********************************/
/* Local, static functions       */
/*********************************/

static void monio_add(const char* svctype, const char* desc, const char* addr_txt, monio_state_t* state_ptr) {
    monio_list.info = realloc(monio_list.info, sizeof(monio_info_t) * (monio_list.count + 1));
    monio_info_t* m = &monio_list.info[monio_list.count++];
    m->svctype = svctype;
    m->desc = desc;
    m->addr = addr_txt;
    m->state_ptr = state_ptr;
}

static bool bad_res_opt(const char* key, unsigned klen V_UNUSED, const vscf_data_t* d V_UNUSED, void* data) {
    log_fatal("plugin_multifo: resource '%s': bad option '%s'", (const char*)data, key);
}

static bool config_res(const char* resname, unsigned resname_len V_UNUSED, const vscf_data_t* opts, void* data) {
    unsigned* residx_ptr = data;
    unsigned res = (*residx_ptr)++;
    mfo_resource_t* this_res = &resources[res];
    this_res->name = strdup(resname);

    if(vscf_get_type(opts) != VSCF_HASH_T)
        log_fatal("plugin_multifo: resource %s: value must be a hash", resname);

    const char* svctype = NULL;
    const vscf_data_t* svctype_data = vscf_hash_get_data_byconstkey(opts, "service_type", true);
    if(svctype_data) {
        if(VSCF_SIMPLE_T != vscf_get_type(svctype_data))
            log_fatal("plugin_multifo: resource %s: 'service_type' must be a string", resname);
        svctype = vscf_simple_get_data(svctype_data);
    }

    unsigned num_addrs;
    const vscf_data_t* addrs_data = vscf_hash_get_data_byconstkey(opts, "addrs", true);
    if(VSCF_HASH_T != vscf_get_type(addrs_data) || !(num_addrs = vscf_hash_get_len(addrs_data)))
        log_fatal("plugin_multifo: resource %s: 'addrs' must be a hash of one or more desc => IPv4 mappings", resname);

    this_res->count = num_addrs;
    this_res->addrstates = malloc(num_addrs * sizeof(addrstate_t));

    for(unsigned i = 0; i < num_addrs; i++) {
        const char* addr_desc = vscf_hash_get_key_byindex(addrs_data, i, NULL);
        const vscf_data_t* addr_data = vscf_hash_get_data_byindex(addrs_data, i);
        if(!vscf_is_simple(addr_data))
            log_fatal("plugin_multifo: resource %s: address %s: all addresses must be strings", resname, addr_desc);
        const char* addr_txt = vscf_simple_get_data(addr_data);

        addrstate_t* as = &this_res->addrstates[i];
        struct in_addr a;
        if(inet_pton(AF_INET, addr_txt, &a) < 1)
            log_fatal("plugin_multifo: resource %s: address %s: parsing '%s' as IPv4 address failed", resname, addr_desc, addr_txt);
        as->ipv4 = a.s_addr;

        char *complete_desc = malloc(strlen(resname) + strlen(addr_desc) + 2);
        strcpy(complete_desc, resname);
        strcat(complete_desc, "/");
        strcat(complete_desc, addr_desc);
        monio_add(svctype, complete_desc, addr_txt, &as->state);
    }

    vscf_hash_iterate(opts, true, bad_res_opt, (void*)resname);
    return true;
}

/*********************************/
/* Exported callbacks start here */
/*********************************/

F_NONNULL
monio_list_t* plugin_multifo_load_config(const vscf_data_t* config) {
    assert(config);
    assert(vscf_get_type(config) == VSCF_HASH_T);

    num_resources = vscf_hash_get_len(config);
    resources = malloc(num_resources * sizeof(mfo_resource_t));
    unsigned residx = 0;
    vscf_hash_iterate(config, false, config_res, &residx);

    return &monio_list;
}

unsigned plugin_multifo_map_resource_dyna(const char* resname) {
    if(!resname)
        log_fatal("multifo plugin requires a resource name");

    for(unsigned i = 0; i < num_resources; i++)
        if(!strcmp(resname, resources[i].name))
            return i;

    log_fatal("Unknown multifo plugin resource '%s'", resname);
}

void plugin_multifo_resolve_dynaddr(unsigned threadnum V_UNUSED, unsigned resnum, const anysin_t* client V_UNUSED, dynaddr_result_t* result) {
    bool all_up = true;
    mfo_resource_t* res = &resources[resnum];

    // Add any UP/DANGER IPs to the results, and track whether
    //  there were any DANGER/DOWN at all or not.
    for(unsigned i = 0; i < res->count; i++) {
        addrstate_t* as = &res->addrstates[i];
        monio_state_uint_t state = monio_state_get(&as->state);
        if(state != MONIO_STATE_UP)
            all_up = false;
        if(state != MONIO_STATE_DOWN)
            result->addrs_v4[result->count_v4++] = as->ipv4;
    }

    // All were down, so just return all (better than empty)
    if(!result->count_v4)
        for(unsigned i = 0; i < res->count; i++)
            result->addrs_v4[result->count_v4++] = res->addrstates[i].ipv4;

    // Cut TTL in half if any were in DOWN or DANGER states
    if(!all_up)
        result->ttl >>= 1;
}
